/*
 * deathmatch.cpp
 *
 *  Created on: 13-09-2013
 *      Author: crm
 */

#include "deathmatch.h"

#include "log.h"

#include "render.h"

#include "gameplay/character.h"
#include "gameplay/arena.h"

using namespace GameModes;

void Deathmatch::init()
	{
	clear();

	timercheck=0.0f;
	timerrestart=0.0f;
	}

void Deathmatch::update(float dt)
	{
	GameMode::update(dt);

	//Log::info("%f %f", timercheck, timerrestart);

	if(timercheck>=0.0f)
		{
		timercheck+=dt;

		if(timercheck>0.5f)
			{
			timercheck=0.0f;

			int alive=0;
			for(std::vector<Character *>::iterator it=chars.begin(); it!=chars.end(); ++it)
				{
				Character *c=*it;

				if(c->getHp()>0.0f)
					++alive;
				}

			if(alive<=1 && chars.size()>1u)
				{
				for(std::vector<Character *>::iterator it=chars.begin(); it!=chars.end(); ++it)
					(*it)->disableControl();

				timercheck=-1.0f;
				timerrestart=0.0f;
				}
			}
		}
	else
		{
		if(timerrestart<1.0f)
			{
			timerrestart+=dt;

			if(timerrestart>=1.0f)
				{
				for(std::vector<Character *>::iterator it=chars.begin(); it!=chars.end(); ++it)
					{
					(*it)->revive();
					(*it)->disableControl();
					}
				}
			}
		else if(timerrestart<2.0f)
			timerrestart+=dt;
		else
			{
			timercheck=0.0f;

			for(std::vector<Character *>::iterator it=chars.begin(); it!=chars.end(); ++it)
				(*it)->enableControl();
			}
		}
	}

void Deathmatch::print()
	{
	GameMode::print();

	Render::setCamera(Render::CAMERA_UI);

	const Render::Camera* acam=arena->getCamera();

	//Render::setMode(Render::DRAW_PRIMITIVE_OVERLAY);
	for(unsigned i=0u; i<chars.size(); ++i)
		{
		const Character *c=chars[i];
		const AVector scrpos=acam->getPointOnScreen(c->getPos());

		const float hppct=std::max(c->getHp(), 0.0f)/c->getStats().maxHP;
		const float mppct=c->getMp()/c->getStats().maxMP;

		const float margin=4;
		const float hpw=80;
		const float hph=6;
		const float mpw=80;
		const float mph=4;

		const AVector hppos=scrpos + AVector(-hpw/2, -margin);
		const AVector mppos=scrpos + AVector(-mpw/2, -margin-hph);

		// HP
		Render::setColor(0.75f, 1.00f, 0, 0);
		Render::drawRect(hppos, hppos+AVector(hpw*hppct, 0), hppos+AVector(hpw*hppct, -hph), hppos+AVector(0, -hph));
		// HP - dopelnienie
		Render::setColor(0.50f, 0.50f, 0, 0);
		Render::drawRect(hppos+AVector(hpw*hppct, 0), hppos+AVector(hpw, 0), hppos+AVector(hpw, -hph), hppos+AVector(hpw*hppct, -hph));

		// MP
		Render::setColor(0.75f, 0, 0.50f, 1.00f);
		Render::drawRect(mppos, mppos+AVector(mpw*mppct, 0), mppos+AVector(mpw*mppct, -mph), mppos+AVector(0, -mph));
		// MP - dopelnienie
		Render::setColor(0.50f, 0, 0.25f, 0.50f);
		Render::drawRect(mppos+AVector(mpw*mppct, 0), mppos+AVector(mpw, 0), mppos+AVector(mpw, -mph), mppos+AVector(hpw*mppct, -mph));
		}

	Render::setCamera(Render::CAMERA_DEFAULT);
	}

void Deathmatch::clear()
	{
	GameMode::clear();

	if(serialized)
		delete [] serialized;

	serialized=NULL;
	}

void Deathmatch::start()
	{
	started=true;
	}

char* Deathmatch::serialize(int &size)
	{
	size=2*sizeof(float);

	if(!serialized)
		serialized=new char[size];

	float *dm=(float *)serialized;

	dm[0]=timercheck;
	dm[1]=timerrestart;

	return serialized;
	}

void Deathmatch::deserialize(char *buff)
	{
	float *dm=(float *)buff;

	timercheck=dm[0];
	timerrestart=dm[1];
	}
