/*
 * arena.h
 *
 *  Created on: 03-09-2013
 *      Author: crm
 */

#pragma once

#include <string>
#include <vector>
#include <list>

#include "asmmath.h"
#include "lua2.h"

#include "camera.h"
#include "graphics/spriteptr.h"

#include "base/eventlistener.h"
#include "base/serializable.h"
#include "base/eventprovider.h"
#include "gameplay/gameobject.h"
#include "objecttype.h"
#include "character.h"
#include "idea.h"
#include "effect.h"

class GameMode;

class Arena: public ObjectType
	{
	public:
		struct Collision
			{
			AVector dir;
			float distance;	// Jak bardzo obiekty w siebie wchodza
			Gameplay::GameObject *a;
			Gameplay::GameObject *b;
			};

	protected:
		struct BackgroundObject
			{
			BackgroundObject(const AVector& pos, /*const Orientation& ori,*/ const std::string& name):
				spr(name), pos(pos)/*, ori(ori)*/ {}

			Graphics::SpritePtr spr;

			AVector pos;
			/*Orientation ori;*/
			};

		AVector alb, art;
		Lua lua; // xxx do wywalenia

		//GameMode *gamemode;

		Graphics::SpritePtr background;
		Graphics::SpritePtr floor;

		std::vector<BackgroundObject *> bgobjs;

		std::list<Gameplay::GameObject *> objects;

		Render::Camera camera;

		std::vector<AVector> positions;

		std::string path;

	public:
		Arena(): ObjectType(), alb(), art() {}
		virtual ~Arena() {clear();}

		virtual bool init(const std::string& name);
		virtual void update(float dt);
		virtual void print();
		virtual void clear();

		void shake(float force);

		void addBackdrop(AVector pos, const std::string& name);
		void addStartPos(AVector pos);

		void setBackground(const std::string& name) {background.setSprite(name);}
		void setFloor(const std::string& name) {floor.setSprite(name);}
		void setSize(float x, float y, float w, float h);
		//void setGameMode(GameMode *s) {gamemode=s;}

		bool getCollision(const AVector& pos) const;

		inline std::string getPath() const {return path;}
		inline const Graphics::SpritePtr& getBackground() const {return background;}
		inline Render::Camera* getCamera() {return &camera;}
		//inline GameMode* getGamemode() {return gamemode;}
		inline AVector getStartPos(int i) const {return (i<0 || positions.size()<1)?(art-alb)/2:positions[i%positions.size()];}
		inline AVector getSize() const {return art-alb;}
		inline AVector getCenter() const {return alb+getSize()/2;}

		AVector getGravity() const;

		Lua& getLua() {return lua;}
	};
