/*
 * effects.cpp
 *
 *  Created on: 20-07-2013
 *      Author: crm
 */

#include "effects.h"

#include <cstdlib>

#include "log.h"

#include "render.h"

using namespace Effects;

/******************************************************************************/
/****************** Slide *****************************************************/
/******************************************************************************/
void Slide::init(const std::string& name, int timeout)
	{
	Effect::init(name, timeout);

	init();
	}

void Slide::init()
	{
	if(spr.getSprite()==NULL)
		return;

	const int W=	spr.getW();
	const int H=	spr.getH();
	const int SX=	spr.getHotspotX();
	const int SY=	spr.getHotspotY();

	const AVector WINDOW=Render::getWindowSize();

	switch(type)
		{
		case LTR:
			startpos=	AVectorSet(-W+SX, SY, 0.0f, 0.0f);
			endpos=		AVectorSet(WINDOW.x+SX,	SY, 0.0f, 0.0f);
		break;

		case RTL:
			startpos=	AVectorSet(WINDOW.x+SX,	SY, 0.0f, 0.0f);
			endpos=		AVectorSet(-W+SX, SY, 0.0f, 0.0f);
		break;

		case TTB:
			startpos=	AVectorSet(SX, -H+SY, 0.0f, 0.0f);
			endpos=		AVectorSet(SX, WINDOW.y+SY,	0.0f, 0.0f);
		break;

		case BTT:
			startpos=	AVectorSet(SX, WINDOW.y+SY,	0.0f, 0.0f);
			endpos=		AVectorSet(SX, -H+SY, 0.0f, 0.0f);
		break;

		default:
			//
		break;
		}
	}

void Slide::update(float dt)
	{
	Effect::update(dt);
	}

void Slide::print()
	{
	AVector pos;

	if(fun==LINEAR)
		{
		pos=AVectorAdd(
			startpos,
			AVectorMultiply(
				AVectorSubtract(
					endpos,
					startpos),
				AVectorReplicate(getPct())) );
		}
	else
		{
		const float  PCT=getPct()-0.5f;
		const float SPCT=((PCT*PCT*PCT)/(0.5f*0.5f*0.5f)+1.0f)/2.0f;

		pos=AVectorAdd(
			startpos,
			AVectorMultiply(
				AVectorSubtract(
					endpos,
					startpos),
				AVectorReplicate(SPCT)) );
		}

	Render::print(spr, Orientation(AVector(pos.x, pos.y, pos.z)));
	}


/******************************************************************************/
/****************** Particle **************************************************/
/******************************************************************************/
void Particle::update(float dt)
	{
	Effect::update(dt);

	AVector dtv=AVectorReplicate(dt);

	spd=AVectorAdd(
		spd,
		AVectorMultiply(
			gravity,
			dtv));

	pos=AVectorAdd(
		pos,
		AVectorMultiply(
			spd,
			dtv));
	}

void Particle::print()
	{
	Render::print(spr, Orientation(AVector(pos.x, pos.y, pos.z), Orientation::FLAT_XZ), 1.0f-getPct());
	}

void Particle::setSpd(float ang, float pow)
	{
	// 0 -> prawo
	// 90 -> gora
	float s, c;

	sincos(ang, s, c);

	//spd.x=c*pow;
	//spd.y=s*pow;
	//spd.z=0.0f;
	//spd.w=0.0f;

	spd=AVectorSet(c*pow, s*pow, 0.0f, 0.0f);
	}

/******************************************************************************/
/****************** Text particle *********************************************/
/******************************************************************************/
void TextParticle::init(const std::string& name, int timeout)
	{
	Particle::init(name, timeout);

	/*str.setAlignCenter();
	str.setAlignMiddle();
	str.setOverflow();
	str.setWrapOff();*/
	}

void TextParticle::print()
	{
	//const float ALPHA=255u*(1.0f-getPct());

	Render::print(spr, Orientation(AVector(pos.x, pos.y, pos.z), Orientation::FLAT_XZ), 1.0f-getPct());
	//str.setAlpha(ALPHA);
	//str.print(pos.x, pos.y-pos.z, 0.0f, ALPHA);
	}

void TextParticle::clear()
	{
	Particle::clear();
	//str.clear();
	}

void TextParticle::setFont(const std::string& font)
	{
	//str.init(font, str.getStr(), spr.getW(), spr.getH());
	/*str.setDimension(spr.getW(), spr.getH());
	str.setFont(font);
	str.init();*/
	}

/******************************************************************************/
/****************** Speech bubble *********************************************/
/******************************************************************************/

void SpeechBubble::init(const std::string& name, int timeout)
	{
	Effect::init(name, timeout);

	spr.setAnim(rand()%spr->getAnims());
	}

void SpeechBubble::update(float dt)
	{
	Effect::update(dt);
	spr.update(dt);
	}

void SpeechBubble::print()
	{
	const float PCT=getPct();
	//AVector pos=AVectorZero();
	AVector pos=c->getActPos();

	float alpha;

	if(PCT<0.25f)
		alpha=getPct()*4.0f;
	else /*if(PCT>0.75f)*/
		alpha=1.0f-getPct()*4.0f;
	Render::print(spr, Orientation(AVector(pos.x, pos.y, pos.z), Orientation::FLAT_XZ), alpha);
	}







