/*
 * main.cpp
 *
 *  Created on: 14-04-2013
 *      Author: crm
 */

extern "C"
	{
	#include <SDL/SDL.h>
	#include <SDL/SDL_image.h>
	#include <SDL/SDL_net.h>
	#undef main
	}

#include "log.h"
#include "asmmath.h"
#include "resfile.h"
#include "resource.h"
#include "databasemanager.h"

#include "render.h"
#include "camera.h"
#include "sound.h"
#include "ui/text.h"
#include "graphics/imagemanager.h"
#include "graphics/spritemanager.h"

#include "consts.h"
#include "clock.h"
#include "gameengine.h"
#include "database/controllermanager.h"
#include "states/intro.h"
//#include "states/demomenu.h"

#include "multiplayer/server/server.h"
#include "multiplayer/client/client.h"

#include "gamemodes/deathmatch.h"
#include "gameplay/arena.h"
#include "database/characterdb.h"
#include "database/arenadb.h"

#include "colliders/box.h"
#include "colliders/collision.h"

SDL_Event e;

Uint8 *key=SDL_GetKeyState(NULL);

float delta=0.0f;


int main(int argc, char *argv[])
	{
	SDL_Init(SDL_INIT_EVERYTHING);

	Log::init();
	Log::setLogLvl(Log::ALL|Log::DEBUG);

	Log::info("Start");
	Log::info("build %s %s", __DATE__, __TIME__);

	Resource::includeList("data.ini");
	Render::init(SCREENWIDTH, SCREENHEIGHT, GAMENAME);

	// Loading~
		{
		//Render::Camera cam(Render::Camera::GUI, 0, 0, Render::getWindowSize().x, Render::getWindowSize().y, -1, 1, 0.0f, 1.0f, false);
		//cam.look(AVector(0, 0, 0), AVector(0, 0, 1), AVector(0, -1, 0));
		//Render::setCamera(&cam);
		Log::debug("Wczytywanie...");
		Render::setCamera(Render::CAMERA_UI);
		Render::setColor(1, 0, 0, 0);
		Render::drawRect(AVector(0, 0), AVector(Render::getWindowSize().x, 0), Render::getWindowSize(), AVector(0, Render::getWindowSize().y));
		UI::Text loading(DEFAULT_FONT, "Wczytywanie...", Render::getWindowSize().x, Render::getWindowSize().y);
		loading.setAlignCenter();
		loading.setAlignMiddle();
		loading.update();
		loading.print(Orientation(AVector(0, Render::getWindowSize().y, 0)), 1.0f);
		Render::update();

		Log::debug("Startowanie");
		SDL_Delay(100);
		}

	SDLNet_Init();
	GameEngine::init();
	Sound::init(32);

	Sound::music("music/song14.ogg", 1.0f);

	CharacterDatabase::init("character/");
	CharacterDatabase::load("download/characters.nvo");

	ArenaDatabase::init("arena/");
	ArenaDatabase::load("download/arenas.nvo");

	Log::info("main: init %ums", SDL_GetTicks());
	Log::info("------------------------------------------------------------");

	const int CLIENTS=2;

	Network::Server s;
	Network::Client c[CLIENTS];

	GameModes::Deathmatch *gm=new GameModes::Deathmatch();
	Arena *a=ArenaDatabase::createByPath("arena/osmiornica.crs", Arena::OBJECT_MULTIPLAYER_SERVER);

	if(!a)
		return 0;

	gm->init();

	s.init(gm, a, 16700);
	for(int i=0; i<CLIENTS; ++i)
		{
		c[i].init("127.0.0.1", 16700);
		c[i].setCharacter(CharacterDatabase::createByPath("character/default.crs"));
		c[i].setController(ControllerManager::getController(i!=0));
		}

	Network::Client *screen=&c[0];

	float lfps, ltime, ltimenvs;
	lfps=0.0f; ltime=0.0f; ltimenvs=0.0f;
	UI::Text info(DEFAULT_FONT, "Plx w8", SCREENWIDTH, 0);
	info.setAlignCenter();
	info.update();

	//Render::Camera *cam;//=c[0].getArena()->getCamera();

	unsigned fpsTimer=		SDL_GetTicks();
	unsigned fpsFrames=		0u;
	unsigned fpsTimerNoVS=	0u;

	unsigned frameTime;
	bool run=true;
	while(run)
		{
		frameTime=SDL_GetTicks();

		//cam=c[0].getArena()->getCamera();

		while(SDL_PollEvent(&e))
			{
			if(e.type==SDL_QUIT || (e.type==SDL_KEYDOWN && e.key.keysym.sym==SDLK_ESCAPE))
				run=false;

			/*if(e.type==SDL_MOUSEMOTION)
				{
				if(!(SDL_GetMouseState(NULL, NULL)&SDL_BUTTON(3)))
					break;
				cam->rotateZ(-e.motion.xrel*delta*10);
				cam->rotateRight(-e.motion.yrel*delta*10);
				}*/
			}

		/*if(SDL_GetMouseState(NULL, NULL)&SDL_BUTTON(3))
			{
			SDL_EventState(SDL_MOUSEMOTION, SDL_DISABLE);
			SDL_WarpMouse(SCREENWIDTH/2, SCREENHEIGHT/2);
			SDL_EventState(SDL_MOUSEMOTION, SDL_ENABLE);
			}

		if(key[SDLK_w]) cam->moveForward( (128+512*key[SDLK_LSHIFT])*delta);
		if(key[SDLK_s]) cam->moveForward(-(128+512*key[SDLK_LSHIFT])*delta);
		if(key[SDLK_a]) cam->moveRight(-(128+512*key[SDLK_LSHIFT])*delta);
		if(key[SDLK_d]) cam->moveRight( (128+512*key[SDLK_LSHIFT])*delta);
		//if(key[SDLK_SPACE]) cam.move(AVector(0, 0,  (128+512*key[SDLK_LSHIFT]))*delta);
		//if(key[SDLK_LCTRL]) cam.move(AVector(0, 0, -(128+512*key[SDLK_LSHIFT]))*delta);
		if(key[SDLK_SPACE]) cam->moveUp( (128+512*key[SDLK_LSHIFT])*delta);
		if(key[SDLK_LCTRL]) cam->moveUp(-(128+512*key[SDLK_LSHIFT])*delta);
		if(key[SDLK_q]) cam->rotateForward( 45*delta);
		if(key[SDLK_e]) cam->rotateForward(-45*delta);
		if(key[SDLK_1]) cam->setType(cam->ORTHOGRAPHIC);
		if(key[SDLK_2]) cam->setType(cam->PERSPECTIVE);*/

		s.update(delta);
		for(int i=0; i<CLIENTS; ++i)
			c[i].update(delta);

		s.print();

		if(key[SDLK_LEFTBRACKET])
			screen=&c[0];
		else if(key[SDLK_RIGHTBRACKET])
			screen=&c[1];

		//Render::setMode(Render::DRAW_2D_OVERLAY);
		Render::setCamera(Render::CAMERA_DEFAULT);
		screen->print();

		//Render::setMode(Render::DRAW_2D_OVERLAY);
		Render::setCamera(Render::CAMERA_UI);
		info.print(Orientation(AVector(0, SCREENHEIGHT-16, 0)), 0.5f);

		fpsTimerNoVS+=SDL_GetTicks()-frameTime;

		Render::update();

		delta=(SDL_GetTicks()-frameTime)/1000.0f;
		if(delta<DELTA_MIN)
			{
			SDL_Delay(1);
			delta=(SDL_GetTicks()-frameTime)/1000.0f;
			}
		else if(delta>DELTA_MAX)
			delta=DELTA_MAX;

		++fpsFrames;
		if(SDL_GetTicks()>fpsTimer+1000u)
			{
			char infobuff[128];
			sprintf(infobuff, "FPS: %.02f;\ttime: %.02f (%.02f);\tdelta: %f", lfps, ltime, ltimenvs, delta);
			info.setStr(infobuff);
			info.update();
			//Log::info("%s", infobuff);

			lfps=fpsFrames*1000.0f/(SDL_GetTicks()-fpsTimer);
			ltime=(SDL_GetTicks()-fpsTimer)/fpsFrames;
			ltimenvs=(float)fpsTimerNoVS/fpsFrames;
			//lflush=Render::fcount;

			fpsTimer=SDL_GetTicks();
			fpsTimerNoVS=0u;
			fpsFrames=0u;
			}
		}

	Log::info("------------------------------------------------------------");
	Log::info("Sprzatanie");

	s.clear();
	for(int i=0; i<CLIENTS; ++i)
		c[i].clear();

	CharacterDatabase::clear();
	ArenaDatabase::clear();
	ControllerManager::clear();
	DatabaseManager::clear();
	Graphics::SpriteManager::clear();
	Graphics::ImageManager::clear();
	Sound::clear();
	SDLNet_Quit();
	Render::clear();
	Resource::clear();

	Log::info("Stop");
	Log::close();

	SDL_Quit();

	return 0;
	}
