/*
 * clientdialogi.cpp
 *
 *  Created on: 21-08-2013
 *      Author: crm
 */

#include "client.h"

#include "log.h"

#include "../packet_dialogi.h"

#include "base/controller.h"
#include "base/gamemode.h"
#include "controllers/dummy.h"
#include "gameplay/arena.h"
#include "gameplay/character.h"

using namespace Network;

const float RECONNECT_DELAY=5.0f;
const float PACKET_RESEND_DELAY=1.0f;

bool Client::init(const std::string& saddr, int sport)
	{
	clear();

	addr=saddr;
	port=sport;

	state=NOT_CONNECTED;
	delay=RECONNECT_DELAY;	// Zeby polaczyl od razu
	reconnects=0;

	self.pid=-1;

	arena=new Arena();
	arena->setType(Arena::OBJECT_MULTIPLAYER_CLIENT);

	return true;
	}

void Client::update(float dt)
	{
	//Log::info("state %d; delay %f; connected %d", state, delay, client.isConnected());

	if(state!=NOT_CONNECTED)
		if(!client.isOk())
			state=NOT_CONNECTED;

	if(state==NOT_CONNECTED)
		{
		if(delay<RECONNECT_DELAY)
			delay+=dt;
		else
			{
			delay=0.0f;
			if(connect())
				{
				reconnects=0;
				state=NEGOTIATION;
				sendWelcome();
				}
			else
				++reconnects;
			}
		}
	else if(state==NEGOTIATION)
		{
		if(arena && gamemode)
			{
			sendInfoCharacter();
			state=READY;
			}
		}

	//client.update();
	while(client.hasPacket())
		parse(client.getPacket());

	if(state!=READY)
		return;

	updateControls(dt);

	if(!gamemode->isStarted())
		gamemode->start();

	gamemode->update(dt);
	}

void Client::updateControls(float dt)
	{
	int tctrl=0;

	ctrl->update(dt);

	tctrl|=ctrl->isPressed(Controller::UP)*ControllerState::UP;
	tctrl|=ctrl->isPressed(Controller::DOWN)*ControllerState::DOWN;
	tctrl|=ctrl->isPressed(Controller::LEFT)*ControllerState::LEFT;
	tctrl|=ctrl->isPressed(Controller::RIGHT)*ControllerState::RIGHT;
	tctrl|=ctrl->isPressed(Controller::A)*ControllerState::A;
	tctrl|=ctrl->isPressed(Controller::B)*ControllerState::B;

	if(tctrl==ctrlState)
		return;

	ctrlState=tctrl;

	sendEventController();
	}

void Client::parse(Packet *pack)
	{
	switch(pack->type)
		{
		case WELCOME_RESPONSE:	parseWelcomeResponse(pack); break;
		case INFO_CHARACTER:	parseInfoCharacter(pack); break;
		case INFO_SERVER:		parseInfoServer(pack); break;
		case INFO_ARENA:		parseInfoArena(pack); break;
		case CHARACTER_ADD:		parseCharacterAdd(pack); break;
		case CHARACTER_DEL:		parseCharacterDel(pack); break;
		case STATE_CHARACTER:	parseStateCharacter(pack); break;
		//case STATE_ARENA:		parseStateArena(pack); break;
		case STATE_GAMEMODE:	parseStateGamemode(pack); break;
		case EVENT_CHARACTER:	parseEventCharacter(pack); break;
		case EVENT_GAMEMODE:	parseEventGamemode(pack); break;
		case MISC_PLAY_SOUND:	parseMiscPlaySound(pack); break;
		case MISC_PLAY_MUSIC:	parseMiscPlayMusic(pack); break;
		case REQUEST:			parseRequest(pack); break;

		case DATA_INFO:
		case DATA:
			transfer.parse(pack);
		break;

		default:
			Log::info("Client.parse: Nieznany typ pakietu (type %d, size %u)", pack->type, pack->length);
		break;
		}

	delete [] pack;
	}

void Client::print()
	{
	if(state!=READY)
		return;
	if(gamemode)
		gamemode->print();
	}

void Client::clear()
	{
	disconnect();

	if(gamemode)
		delete gamemode;
	if(arena)
		delete arena;
	if(self.character)
		delete self.character;

	for(unsigned i=0; i<chars.size(); ++i)
		delete chars[i];

	chars.clear();

	gamemode=NULL;
	arena=NULL;

	self=Player();
	ctrl=NULL;
	ctrlState=0;

	delay=0.0f;
	}

bool Client::connect()
	{
	return client.init(addr, port);
	}

void Client::disconnect()
	{
	transfer.clear();

	if(client.isOk())
		send(BYE, 0, NULL);
	client.disconnect();

	client.clear();
	self.pid=-1;
	}


void Client::setController(Controller *sctrl)
	{
	ctrl=sctrl;
	}

void Client::setCharacter(Character *c)
	{
	if(self.character)
		delete self.character;
	self.character=c;

	//self.character->setType(Character::OBJECT_NETWORK_CLIENT_SIDE); 	// zbedne, i tak juz bednie init'niety

	//sendCharacterInfo();
	}

void Client::send(int type, int length, const char *data)
	{
	//Log::info("Client.send: type %d, len %d", type, length);
	client.send(type, length, data);
	}

/*void Client::sendPlayerInfo()
	{
	Connect ci;

	ci.id=-1;
	ci.pid=self.pid;

	send(CONNECT, sizeof(ci), (const char *)&ci);
	}

void Client::sendCharacterInfo()
	{
	int size;

	if(!self.character)
		return;
	if( self.character->getPath().empty())
		return;
	if(!self.getState(Player::CONNECTED))
		return;


	Character *c=self.character;

	size=sizeof(CharacterInfo)+c->getPath().length()+1;

	CharacterInfo *ci=(CharacterInfo *)new char[size];

	ci->id=-1;
	strcpy(ci->path, c->getPath().c_str());

	Log::info("Client.sendCharacterInfo: CHARACTER_INFO: path \"%s\"", ci->path);

	send(CHARACTER_INFO, size, (const char *)ci);

	self.addState(Player::HAS_CHARACTER);

	delete [] ci;	// alokujesz tablice, deletujesz tablice
	}*/

/*Controller* Client::getController(int i) const
	{
	return players[i]->ctrl;
	}

Character* Client::getCharacter(int i) const
	{
	return players[i]->character;
	}*/


void Client::addCharacter(Character *c)
	{
	if(!c)
		return;
	//if(getCharacter(c->getId()))
	//	return;

	/*for(unsigned i=0u; i<chars.size(); ++i)
		{
		if(chars[i])
			continue;

		chars[i]=c;
		break;
		}*/

	Log::info("Client[%2d].addCharacter: Dodawanie postaci, cid %d", self.pid, c->getId());

	if((int)chars.size()<=c->getId())
		{
		const unsigned osize=chars.size();

		Log::debug("Client[%2d].addCharacter: Powiekszanie tablicy, osize: %u", self.pid, osize);

		chars.resize(c->getId()+1);

		for(unsigned i=osize; i<chars.size(); ++i)
			chars[i]=NULL;
		}

	if(chars[c->getId()])
		{
		Log::info("Client[%2d].addCharacter: Jest juz postac o id %d, kasowanie", self.pid, c->getId());

		gamemode->delCharacter(chars[c->getId()]);

		delete chars[c->getId()];
		}

	chars[c->getId()]=c;

	gamemode->addCharacter(c);

	Log::info("Client[%2d].addCharacter: Postac dodana, cid %d", self.pid, c->getId());
	}

Character* Client::getCharacter(int cid)
	{
	if(cid<0 || cid>=(int)chars.size())
		return NULL;

	return chars[cid];
	}
