/*
 * client_parser.cpp
 *
 *  Created on: 08-09-2013
 *      Author: crm
 */

#include "client.h"

#include <fstream>

#include "log.h"

#include "sound.h"

#include "../packet_dialogi.h"
#include "base/controller.h"
#include "base/event.h"
#include "base/gamemode.h"
#include "controllers/dummy.h"
#include "database/arenadb.h"
#include "database/characterdb.h"
#include "gamemodes/deathmatch.h"
#include "gameplay/arena.h"
#include "gameplay/character.h"

using namespace Network;

void Client::parseWelcomeResponse(Packet *pack)
	{
	Welcome *w;

	w=(Welcome*)pack->data;

	self.pid=w->pid;

	Log::info("Client.parseWelcomeResponse: pid: %d", self.pid);
	}

void Client::parseInfoCharacter(Packet *pack)
	{
	InfoCharacter *ic=(InfoCharacter*)pack->data;
	Character *c=CharacterDatabase::createByCRC(ic->crc, Character::OBJECT_MULTIPLAYER_CLIENT);

	Log::info("Client.parseInfoCharacter: ID: %d; CRC32: 0x%08X; char: 0x%08X; id %d", ic->cid, ic->crc, c, c?c->getId():-1);

	if(!c)
		{
		Log::info("Client.parseInfoCharacter: Nie znaleziono postaci o CRC32 0x%08X, trzeba ja pobrac...", ic->crc);

		sendRequestCharacter(ic->crc);

		return;
		}

	c->setId(ic->cid);

	addCharacter(c);
	}

void Client::parseInfoServer(Packet *pack)
	{
	//InfoServer *is=(InfoServer*)pack->data;
	// is->gamemode

	if(gamemode)
		delete gamemode;
	gamemode=new GameModes::Deathmatch();

	if(arena)
		gamemode->setArena(arena);
	}

void Client::parseInfoArena(Packet *pack)
	{
	InfoArena *ia=(InfoArena*)pack->data;

	Arena *a=ArenaDatabase::createByCRC(ia->crc, Arena::OBJECT_MULTIPLAYER_CLIENT);

	if(!a)
		{
		Log::info("Client.parseInfoArena: Nie znaleziono areny o CRC32 0x%08X, trzeba ja pobrac...", ia->crc);

		sendRequestArena(ia->crc);

		return;
		}

	if(arena)
		delete arena;
	arena=a;
	arena->setType(Arena::OBJECT_MULTIPLAYER_CLIENT);

	if(gamemode)
		gamemode->setArena(arena);
	}

void Client::parseCharacterAdd(Packet *pack)
	{
	Log::info("Client.sendCharacterAdd");
	return;

	AddDelCharacter *adc=(AddDelCharacter*)pack->data;

	if(!arena)
		return;

	Log::info("Client.parseCharacterAdd");

	if(adc->cid<0)
		{
		Log::fail("Client.parseCharacterAdd: Ujemny identyfikator postaci; cid %d", adc->cid, adc->pid);
		return;
		}

	if(adc->cid>=(int)chars.size() || !chars[adc->cid])
		{
		Log::fail("Client.parseCharacterAdd: Nie ma informacji o postaci cid %d", adc->cid, chars.size());
		sendInfoCharacter(adc->cid);
		return;
		}

	//arena->addCharacter(chars[adc->cid]);
	gamemode->addCharacter(chars[adc->cid]);
	}

void Client::parseCharacterDel(Packet *pack)
	{
	AddDelCharacter *adc=(AddDelCharacter*)pack->data;

	if(!arena)
		return;

	if(adc->cid<0)
		{
		Log::fail("Client.parseCharacterDel: Ujemny identyfikator postaci; cid %d", adc->cid, adc->pid);
		return;
		}

	if(adc->cid>=(int)chars.size() || !chars[adc->cid])
		{
		Log::fail("Client.parseCharacterDel: Nie ma informacji o postaci cid %d", adc->cid, chars.size());
		return;
		}

	//arena->delCharacter(chars[adc->cid]);
	gamemode->delCharacter(chars[adc->cid]);

	delete chars[adc->cid];
	chars[adc->cid]=NULL;
	}

void Client::parseStateCharacter(Packet *pack)
	{
	State *s=(State*)pack->data;
	Character *c;

	c=getCharacter(s->id);

	if(!c)
		{
		Log::debug("Client[%2d].parseStateCharacter: Otrzymano state nieznanej postaci %d", self.pid, s->id);

		return;
		}

	c->deserialize(s->data);
	}

/*void Client::parseStateArena(Packet *pack)
	{
	State *s=(State*)pack->data;

	arena->deserialize(s->data);
	}*/

void Client::parseStateGamemode(Packet *pack)
	{
	State *s=(State*)pack->data;

	gamemode->deserialize(s->data);
	}

void Client::parseEventCharacter(Packet *pack)
	{
	EventCharacter *e=(EventCharacter*)pack->data;
	Character *c;

	c=getCharacter(e->id);

	if(!c)
		{
		Log::debug("Client[%2d].parseEventCharacter: Otrzymano event nieznanej postaci %d", self.pid, e->id);

		//sendInfoCharacter(e->id);

		return;
		}

	c->setState(e->state);
	c->setPos(AVector(e->x, e->y, e->z));
	c->setSpd(AVector(e->spdx, e->spdy, e->spdz));
	c->setHp(e->hp);
	c->setMp(e->air);

	switch(e->event)
		{
		case Event::CHARACTER_CHARGED:
			Log::debug("Client.parseEventCharacter: Postac %d castuje pomysl %d", c->getId(), e->datai);

			c->giveIdeaSelf(e->datai);
		break;

		case Event::CHARACTER_CHARGED|Event::CHARACTER_GOT_IDEA:
			Log::debug("Client.parseEventCharacter: Postac %d wycastowala pomysl %d", c->getId(), e->datai);

			c->giveIdeaSelf(e->datai);
		break;

		case Event::CHARACTER_GOT_IDEA:
			Log::debug("Client.parseEventCharacter: Postac %d podniosla pomysl %d", c->getId(), e->datai);

			c->giveIdeaOther(e->datai);
		break;

		case Event::CHARACTER_ATTACK:
			c->attack();
		break;

		case Event::CHARACTER_HIT:
			c->hit(e->dataf);
			c->setHp(e->hp);
		break;

		case Event::CHARACTER_COMBO:
			c->setLuaRef(e->datai);
		break;

		case Event::CHARACTER_SOUND:
			Sound::sound(c->getSound(e->datai), e->dataf);
		break;

		default:
			// c->setDir((Character::Direction)e->datai);
			//Log::info("Client::parseEventCharacter: Nieobslugiwany event (%s: %d)", getEventNameCharacter(e->event).c_str(), e->event);
		break;
		}
	}

void Client::parseEventGamemode(Packet *pack)
	{
	//
	}

void Client::parseMiscPlaySound(Packet *pack)
	{
	//
	}

void Client::parseMiscPlayMusic(Packet *pack)
	{
	//
	}

void Client::parseRequest(Packet *pack)
	{
	Request *r=(Request*)pack->data;

	if(r->type!=Request::CHARACTER)
		{
		Log::fail("Client.parseRequest: Serwer prosil o cos innego niz postac o:");
		return;
		}

	const CharacterDatabase::Entry *cdb;
	cdb=CharacterDatabase::getByCRC(r->crc);

	if(!cdb)
		{
		Log::fail("Client.parseRequest: Nie znaleziono postaci o CRC32 0x%08X", cdb->crc);

		cdb=CharacterDatabase::getByPath(self.character->getPath());
		if(!cdb)
			Log::fail("Client.parseRequest: Nie znaleziono informacji o postaci gracza (\"%s\")", self.character->getPath().c_str());
		else
			Log::fail("Client.parseRequest: CRC32 postaci gracza: 0x%08X", cdb->crc);

		return;
		}

	transfer.upload(Request::CHARACTER, cdb->crc);
	}


