/*
 * main.cpp
 *
 *  Created on: 04-09-2013
 *      Author: crm
 */

#include "server.h"

#include "log.h"

#include "../packet_dialogi.h"
#include "clock.h"
#include "consts.h"
#include "database/characterdb.h"

using namespace Network;

bool Server::init(GameMode *sgamemode, Arena *sarena, int sport)
	{
	if(!sgamemode)
		{
		Log::fail("Server.init: NULL gamemode");
		return false;
		}

	if(!sarena)
		{
		Log::fail("Server.init: NULL arena");
		return false;
		}

	clear();

	server.addEventListener(this);

	port=sport;
	maxclients=sgamemode->getMaxPlayerCount();

	gamemode=sgamemode;
	arena=sarena;

	currentPid=0;
	updateDelay=0.0f;
	gameModeState=false;

	if(!server.init(port, maxclients))
		{
		Log::fail("Server.init: Nie udalo sie uruchomic socketa");
		return false;
		}

	players.resize(maxclients);
	for(unsigned i=0u; i<players.size(); ++i)
		players[i]=NULL;

	gamemode->setArena(arena);

	//arena->addEventListener(this);
	if(arena->getType()!=Arena::OBJECT_MULTIPLAYER_SERVER)
		{
		arena->setType(Arena::OBJECT_MULTIPLAYER_SERVER);
		arena->init(arena->getPath());
		}
	arena->getLua().addRegistry(LUA_PTR_EVENT_LISTENER, this);

	return true;
	}

void Server::update(float dt)
	{
	//server.update();	// odkomentowac tu i zakomentowac w server.run aby dzialalo na tym samym watku.

	if(!server.isListening())
		return;

	server.update();

	updateEvents(dt);
	updatePlayers(dt);
	updateGameplay(dt);
	}

void Server::updateEvents(float dt)
	{
	while(events.size()>0u)
		{
		Event e=events.front();
		events.pop_front();

		if(e.type==Event::TYPE_CHARACTER)
			{
			//if(e.event!=Event::CHARACTER_NONE /*&& !(e.event&Event::CHARACTER_MOVE)*/)

			/*if(e.event==e.CHARACTER_CHARGED|e.CHARACTER_GOT_IDEA)
				{
				// Nowy pomysl, dodaj sobie do listy
				}*/

			// Laczenie eventow w jedno
			const int EVENTS_DONT_COMBINE=e.CHARACTER_CHARGED|e.CHARACTER_GOT_IDEA|e.CHARACTER_ATTACK|e.CHARACTER_HIT|e.CHARACTER_COMBO|e.CHARACTER_SOUND;

			if(e.event&~EVENTS_DONT_COMBINE)
				{
				for(std::list<Event>::iterator it=events.begin(); it!=events.end(); ++it)
					{
					Event &ec=*it;

					if(ec.type!=ec.TYPE_CHARACTER)
						continue;
					if(ec.character.who!=e.character.who)
						continue;

					if(ec.event&~EVENTS_DONT_COMBINE)
						continue;

					//printf("Laczenie %s i %s", getEventNameCharacter(e.event).c_str(), getEventNameCharacter(ec.event).c_str());

					e.event|=ec.event;

					events.erase(it--);
					}
				}

			sendEventCharacter(e.character.who, e);
			}
		else if(e.type==Event::TYPE_IDEA)
			{
			//
			}
		else if(e.type==Event::TYPE_NETWORK)
			{
			if(e.event==Event::NETWORK_CONNECT)
				/**/;
			else if(e.event==Event::NETWORK_DISCONNECT)
				{
				ServerPlayer *sp=findPlayerBySocketID(e.network.id);
				if(sp)
					{
					sp->transfer.clear();
					sp->setState(ServerPlayer::NOT_CONNECTED);
					sp->timetolive=Clock::getTimeMilis()+PLAYER_TIME_TO_LIVE;
					}
				}
			else if(e.event==Event::NETWORK_PACKET)
				{
				while(server.hasPacket(e.network.id))
					parse(e.network.id, server.getPacket(e.network.id));
				}
			}
		}
	}

void Server::updatePlayers(float dt)
	{
	// Connecty rozpatrywane sa dopiero po otrzymaniu pakietu WELCOME

	for(unsigned i=0u; i<players.size(); ++i)
		{
		ServerPlayer *p=players[i];
		if(!p)
			continue;
		if(!p->getState(ServerPlayer::CONNECTED))
			{
			if(p->timetolive<Clock::getTimeMilis() || !p->character)
				{
				Log::info("Server.updatePlayers: Zapominanie o graczu pid %d", p->pid);

				disconnect(p);	// na wszelki wypadek

				delete players[i];
				players[i]=NULL;
				}
			continue;
			}

		p->transfer.update();

		if(p->timetostate<Clock::getTimeMilis())
			{
			sendStateCharacter(p->socketid, NULL);
			//sendStateArena(p->socketid);
			sendStateGamemode(p->socketid);

			p->timetostate=Clock::getTimeMilis()+PLAYER_TIME_TO_STATE+(rand()%PLAYER_TIME_TO_STATE_RANDOM);

			//Log::debug("Server.updatePlayers: Wysylanie stanu gry do gracza %d; czas: %d; nastepny: %d", p->pid, Clock::getTimeMilis(), p->timetostate);
			}

		/*while(server.hasPacket(p->socketid))
			parse(p->socketid, server.getPacket(p->socketid));*/
		}
	}

void Server::updateGameplay(float dt)
	{
	gamemode->update(dt);
	}


void Server::print()
	{
	/*static int tt=0;

	if(tt<Clock::getTimeMilis())
		{
		float dt=(Clock::getTimeMilis()-tt+1000)/1000.0f;

		unsigned s=0;//=server.getSendSum();
		unsigned r=0;//=server.getReadSum();

		for(int i=0; i<server.getClientCount(); ++i)
			{
			SocketClient *client=server.getClient(i);

			if(!client)
				continue;

			s+=client->getSendSum();
			r+=client->getReadSum();
			}

		Log::info("Server.print: dt %fs, read %uB, send %uB", dt, r, s);

		tt=Clock::getTimeMilis()+1000;
		}*/
	}

void Server::clear()
	{
	ServerPlayer *c;

	if(server.isListening())
		{
		broadcast(BYE, 0, NULL);

		server.clear();
		}

	for(unsigned i=0u; i<players.size(); ++i)
		{
		c=players[i];
		players[i]=NULL;

		if(!c)
			continue;

		disconnect(c);
		delete c;
		}
	if(gamemode)
		delete gamemode;
	if(arena)
		delete arena;

	//ctrls.clear();
	//chars.clear();
	players.clear();

	gamemode=NULL;
	arena=NULL;
	}


void Server::parse(int socket, Packet *pack)
	{
	if(!pack)
		return;

	switch(pack->type)
		{
		case WELCOME:
			parseWelcome(socket, pack);
		break;

		case BYE:
			parseBye(socket, pack);
		break;

		case INFO_CHARACTER:
			parseInfoCharacter(socket, pack);
		break;

		case INFO_SERVER:
			parseInfoServer(socket, pack);
		break;

		case INFO_ARENA:
			parseInfoArena(socket, pack);
		break;

		case EVENT_CONTROLLER:
			parseEventController(socket, pack);
		break;

		case REQUEST:
			parseRequest(socket, pack);
		break;

		case DATA_INFO:
		case DATA:
			{
			ServerPlayer *sp=findPlayerBySocketID(socket);
			if(!sp)
				{
				server.disconnect(socket);
				break;
				}

			sp->transfer.parse(pack);
			}
		break;

		default:
			Log::info("Client.parse: Nieobslugiwany typ pakietu (type %d, size %u)", pack->type, pack->length);
		break;
		}

	delete [] pack;
	}


void Server::send(int socket, int type, int length, const char *data)
	{
	//Log::info("Server.send[%02d]: type %d, len %d", socket, type, length);
	server.send(socket, type, length, data);
	}

void Server::broadcast(int type, int length, const char *data)
	{
	ServerPlayer *sp;

	//Log::info("Server.broadcast[--]: type %d, len %d", type, length);

	for(int i=0; i<(int)players.size(); ++i)
		{
		sp=players[i];

		if(!sp)
			continue;

		if(!sp->getState(ServerPlayer::CAN_RECEIVE_BROADCAST))
			continue;

		server.send(sp->socketid, type, length, data);
		}
	}


void Server::setGamemode(GameMode *s)
	{
	gamemode=s;
	}

void Server::setArena(Arena *s)
	{
	arena=s;

	arena->setType(Arena::OBJECT_MULTIPLAYER_SERVER);

	//sendArenaInfo(-1);
	}


Server::ServerPlayer* Server::findPlayerByID(int pid)
	{
	ServerPlayer *p;

	for(unsigned i=0u; i<players.size(); ++i)
		{
		p=players[i];
		if(!p)
			continue;
		if(p->pid==pid)
			return p;
		}

	return NULL;
	}

Server::ServerPlayer* Server::findPlayerBySocketID(int socket)
	{
	ServerPlayer *p;

	for(unsigned i=0u; i<players.size(); ++i)
		{
		p=players[i];
		if(!p)
			continue;
		if(p->socketid==socket)
			return p;
		}

	return NULL;
	}

void Server::disconnect(ServerPlayer *sp)
	{
	if(!sp)
		return;

	if(server.isConnected(sp->socketid))
		{
		send(sp->socketid, BYE, 0, NULL);
		server.disconnect(sp->socketid);
		//sp->socketid->setDNDoff();
		}

	if(sp->character)
		{
		gamemode->delCharacter(sp->character);

		sendCharacterDel(sp->character);

		chars[sp->character->getId()]=NULL;
		delete sp->character;
		}
	if(sp->ctrl)
		delete sp->ctrl;

	sp->transfer.clear();

	sp->character=NULL;
	sp->ctrl=NULL;

	sp->pid=-1;
	sp->socketid=-1;
	sp->setState(ServerPlayer::NOT_CONNECTED);
	}


void Server::addCharacter(Character *c)
	{
	if(!c)
		return;
	if(getCharacter(c->getId()))
		return;

	/*for(unsigned i=0u; i<chars.size(); ++i)
		{
		if(chars[i])
			continue;

		chars[i]=c;
		break;
		}*/

	Log::info("Server.addCharacter: Dodawanie postaci, cid %d", c->getId());

	if(c->getId()<0)
		{
		for(unsigned i=0; i<chars.size(); ++i)
			{
			if(chars[i]!=NULL)
				continue;

			chars[i]=c;
			c->setId(i);
			break;
			}

		if(c->getId()<0)
			{
			chars.push_back(c);
			c->setId(chars.size()-1);
			}
		}
	else
		{
		if((int)chars.size()<=c->getId())
			{
			const unsigned osize=chars.size();

			chars.resize(c->getId()+1);

			for(unsigned i=osize; i<chars.size(); ++i)
				chars[i]=NULL;
			}

		chars[c->getId()]=c;
		}

	//arena->addCharacter(c);
	gamemode->addCharacter(c);

	Log::info("Server.addCharacter: Postac dodana, cid %d", c->getId());
	}

Character* Server::getCharacter(int cid)
	{
	if(cid<0 || cid>=(int)chars.size())
		return NULL;

	return chars[cid];
	}
