/*
 * server_parser.cpp
 *
 *  Created on: 08-09-2013
 *      Author: crm
 */

#include "server.h"

#include "log.h"
#include "resource.h"

#include "../packet_dialogi.h"
#include "clock.h"
#include "consts.h"
#include "database/characterdb.h"
#include "gameplay/character.h"

using namespace Network;

void Server::parseWelcome(int socket, Packet *pack)
	{
	Welcome *w;
	ServerPlayer *p;

	w=(Welcome*)pack->data;
	p=findPlayerBySocketID(socket);

	//if(p)
	//	Log::info("Server.parseWelcome: Ponowne powitanie, pid: %d", p->pid);
	if(!p)
		{
		if(w->pid>=0)
			{
			//Log::info("Server.parseWelcome: Otrzymano istniejacy pid: %d", w->pid);
			p=findPlayerByID(w->pid);

			if(!p)
				w->pid=-1;
			else if(p->getState(ServerPlayer::CONNECTED))
				{
				//Log::info("Server.parseWelcome: pid %d juz jest polaczony, przydzielanie nowego", w->pid);
				w->pid=-1;
				}
			else
				{
				p->socketid=socket;
				//Log::info("Server.parseWelcome: pid %d wrocil do gry", w->pid);
				}
			}
		if(w->pid<0)
			{
			//Log::info("Server.parseWelcome: Nowy gracz");

			for(unsigned i=0u; i<players.size(); ++i)
				{
				if(players[i])
					continue;

				players[i]=new ServerPlayer();

				p=players[i];
				p->pid=currentPid++;
				p->socketid=socket;
				p->ctrl=new ControllerDummy();
				p->setState(ServerPlayer::CONNECTED);
				p->timetostate=Clock::getTimeMilis()+PLAYER_TIME_TO_STATE+(rand()%PLAYER_TIME_TO_STATE_RANDOM);
				break;
				}
			}
		}

	sendWelcomeResponse(socket, p->pid);
	sendInfoServer(socket);
	sendInfoArena(socket);
	sendInfoCharacter(socket, NULL);	// Niby ten null nie jest potrzebny, ale wyobrazilem sobie sytuacje gdzie wywoluje sie funkcja wskaznikowa zamiast intowej...
	sendStateGamemode(socket);
	//sendStateArena(socket);
	sendStateCharacter(socket, NULL);
	}

void Server::parseBye(int socket, Packet *pack)
	{
	server.disconnect(socket);

	disconnect(findPlayerBySocketID(socket));
	}

void Server::parseInfoCharacter(int socket, Packet *pack)
	{
	InfoCharacter *ci=(InfoCharacter*)pack->data;

	ServerPlayer *sp=findPlayerBySocketID(socket);

	if(!sp)
		{
		Log::fail("Server.parseInfoCharacter: Nieznana osoba (klient %d) wysyla postac, ubic ja", socket);
		server.disconnect(socket);
		return;
		}

	if(sp->character)
		{
		/*Log::info("Server.parseInfoCharacter: Gracz %d juz ma jedna postac, wywalanie poprzedniej", p->pid);

		gamemode->delCharacter(p->character);
		delete p->character;
		p->character=NULL;*/

		Character *c=getCharacter(ci->cid);

		if(!c)
			{
			Log::fail("Server.parseInfoCharacter: Gracz %d poprosil o info postaci %d - nie ma takiej", sp->pid, ci->cid);
			return;
			}
		else
			Log::debug("Server.parseInfoCharacter: Gracz %d poprosil o info postaci %d", sp->pid, ci->cid);

		sendInfoCharacter(socket, c);
		return;
		}

	if(ci->crc==0)
		{
		server.disconnect(socket);
		return;
		}

	if(!CharacterDatabase::getByCRC(ci->crc))
		{
		Log::info("Server.parseInfoCharacter: Gracz %d ma postac 0x%08X, pobieranie", sp->pid, ci->crc);
		sendRequestCharacter(socket, ci->crc);
		return;
		}

	Character *c=CharacterDatabase::createByCRC(ci->crc, Character::OBJECT_MULTIPLAYER_SERVER);

	if(!c)
		{
		Log::fail("Server.parseInfoCharacter: Gracz %d ma nieprawidlowa postac (%s)", sp->pid, CharacterDatabase::getByCRC(ci->crc)->path.c_str());
		return;
		}

	c->setController(sp->ctrl);
	c->addEventListener(this);
	c->getLua().addRegistry(LUA_PTR_EVENT_LISTENER, this);

	sp->character=c;
	sp->addState(ServerPlayer::HAS_CHARACTER);

	addCharacter(c);
	sendInfoCharacter(c);
	//sendCharacterAdd(c);
	}

void Server::parseInfoServer(int socket, Packet *pack)
	{
	sendInfoServer(socket);
	}

void Server::parseInfoArena(int socket, Packet *pack)
	{
	sendInfoArena(socket);
	}

void Server::parseEventController(int socket, Packet *pack)
	{
	ServerPlayer *sp=findPlayerBySocketID(socket);
	ControllerState *cs=(ControllerState*)pack->data;

	if(!sp)
		return;

	/*for(unsigned i=0; i<32; ++i)
		fprintf(stdout, "%d", !!(cs->keys&(1<<i)));
	fprintf(stdout, "\n");
	for(unsigned i=0; i<32; ++i)
		fprintf(stdout, "%d", !!((ControllerState::UP|ControllerState::DOWN|ControllerState::LEFT|ControllerState::RIGHT|ControllerState::A|ControllerState::B)&(1<<i)));
	fprintf(stdout, "\n");

	printf("u %d d %d l %d r %d; a %d b %d",
		!!(cs->keys&ControllerState::UP),
		!!(cs->keys&ControllerState::DOWN),
		!!(cs->keys&ControllerState::LEFT),
		!!(cs->keys&ControllerState::RIGHT),
		!!(cs->keys&ControllerState::A),
		!!(cs->keys&ControllerState::B));*/

	sp->ctrl->setKeyState(Controller::UP, cs->keys&ControllerState::UP);
	sp->ctrl->setKeyState(Controller::DOWN, cs->keys&ControllerState::DOWN);
	sp->ctrl->setKeyState(Controller::LEFT, cs->keys&ControllerState::LEFT);
	sp->ctrl->setKeyState(Controller::RIGHT, cs->keys&ControllerState::RIGHT);
	sp->ctrl->setKeyState(Controller::A, cs->keys&ControllerState::A);
	sp->ctrl->setKeyState(Controller::B, cs->keys&ControllerState::B);
	}

void Server::parseRequest(int socket, Packet *pack)
	{
	Request *r=(Request*)pack->data;

	ServerPlayer *sp=findPlayerBySocketID(socket);
	if(!sp)
		{
		server.disconnect(socket);
		return;
		}

	sp->transfer.upload(r->type, r->crc);
	}

