/*
 * server_send.cpp
 *
 *  Created on: 10-09-2013
 *      Author: crm
 */

#include "server.h"

#include "log.h"

#include "../packet_dialogi.h"
#include "database/characterdb.h"
#include "database/arenadb.h"

using namespace Network;

void Server::sendWelcomeResponse(int socket, int pid)
	{
	Welcome w;

	w.pid=pid;

	send(socket, WELCOME_RESPONSE, sizeof(w), (char*)&w);
	}

void Server::sendBye(int socket)
	{
	send(socket, BYE, 0, NULL);
	}

void Server::sendInfoCharacter(Character *c)
	{
	const CharacterDatabase::Entry *cdb;
	cdb=CharacterDatabase::getByPath(c->getPath());

	InfoCharacter ci;

	ci.cid=c->getId();
	ci.crc=cdb->crc;

	broadcast(INFO_CHARACTER, sizeof(ci), (char*)&ci);
	}

void Server::sendInfoCharacter(int socket, Character *c)
	{
	if(!c)
		{
		for(unsigned i=0; i<chars.size(); ++i)
			if(chars[i]!=NULL)							// Upewnij sie, ze nie zrobimy pieknej rekurencji	// Za kazdym razem jak na to patrze to czuje sie dumny z siebie.
				sendInfoCharacter(socket, chars[i]);
		return;
		}

	const CharacterDatabase::Entry *cdb;
	cdb=CharacterDatabase::getByPath(c->getPath());

	InfoCharacter ci;

	ci.cid=c->getId();
	ci.crc=cdb->crc;

	send(socket, INFO_CHARACTER, sizeof(ci), (char*)&ci);
	}

void Server::sendInfoServer(int socket)
	{
	InfoServer is;
	is.gamemode=gamemode->getType();

	send(socket, INFO_SERVER, sizeof(is), (char*)&is);
	}

void Server::sendInfoArena(int socket)
	{
	InfoArena ia;

	const ArenaDatabase::Entry *adb=ArenaDatabase::getByPath(arena->getPath());

	ia.crc=adb->crc;

	send(socket, INFO_ARENA, sizeof(ia), (char*)&ia);
	}

void Server::sendCharacterAdd(Character *c)
	{
	Log::info("Server.sendCharacterAdd");
	return;

	AddDelCharacter adc;

	adc.cid=c->getId();
	adc.pid=-1;

	broadcast(CHARACTER_ADD, sizeof(adc), (char*)&adc);
	}

void Server::sendCharacterDel(Character *c)
	{
	//Log::info("Server.sendCharacterDel: TODO");
	//return;

	AddDelCharacter adc;

	adc.cid=c->getId();
	adc.pid=-1;

	broadcast(CHARACTER_DEL, sizeof(adc), (char*)&adc);
	}

void Server::sendCharacterAdd(int socket, Character *c)
	{
	Log::info("Server.sendCharacterAdd");
	return;

	AddDelCharacter adc;

	adc.cid=c->getId();
	adc.pid=-1;

	send(socket, CHARACTER_ADD, sizeof(adc), (char*)&adc);
	}

void Server::sendCharacterDel(int socket, Character *c)
	{
	//Log::info("Server.sendCharacterDel");
	//return;

	AddDelCharacter adc;

	adc.cid=c->getId();
	adc.pid=-1;

	send(socket, CHARACTER_DEL, sizeof(adc), (char*)&adc);
	}

void Server::sendStateCharacter(int socket, Character *c)
	{
	if(!c)
		{
		for(unsigned i=0; i<chars.size(); ++i)
			if(chars[i]!=NULL)
				sendStateCharacter(socket, chars[i]);
		return;
		}

	int size;
	char *data;

	data=c->serialize(size);

	size+=sizeof(State);

	State *s=(State*)new char[size];

	s->id=c->getId();
	memcpy(s->data, data, size-sizeof(State));

	send(socket, STATE_CHARACTER, size, (char*)s);

	delete [] s;
	}

/*void Server::sendStateArena(int socket)
	{
	int size;
	char *data;

	data=arena->serialize(size);

	size+=sizeof(State);

	State *s=(State*)new char[size];

	s->id=-1;
	memcpy(s->data, data, size-sizeof(State));

	send(socket, STATE_ARENA, size, (char*)s);

	delete [] s;
	}*/

void Server::sendStateGamemode(int socket)
	{
	int size;
	char *data;

	data=gamemode->serialize(size);

	size+=sizeof(State);

	State *s=(State*)new char[size];

	s->id=-1;
	memcpy(s->data, data, size-sizeof(State));

	send(socket, STATE_GAMEMODE, size, (char*)s);

	delete [] s;
	}

void Server::sendEventCharacter(Character *c, const Event& e)
	{
	EventCharacter ec;

	ec.id=c->getId();
	ec.state=c->getState();
	ec.x=c->getPos().x;
	ec.y=c->getPos().y;
	ec.z=c->getPos().z;
	ec.spdx=c->getSpd().x;
	ec.spdy=c->getSpd().y;
	ec.spdz=c->getSpd().z;
	ec.hp=c->getHp();
	ec.air=c->getMp();
	ec.event=e.event;
	ec.datai=e.character.datai;
	ec.dataf=e.character.dataf;

	//Log::info("Server.sendEventCharacter: %d: %d, %d; %f %d", ec.id, ec.event, ec.state, ec.dataf, ec.datai);

	/*fprintf(stdout, " > ");
	if(e.event==Event::CHARACTER_NONE) fprintf(stdout, "[CHARACTER_NONE]");
	if(e.event&Event::CHARACTER_MOVE) fprintf(stdout, "[CHARACTER_MOVE]");
	if(e.event&Event::CHARACTER_MOVE_LEFT) fprintf(stdout, "[CHARACTER_MOVE_LEFT]");
	if(e.event&Event::CHARACTER_MOVE_RIGHT) fprintf(stdout, "[CHARACTER_MOVE_RIGHT]");
	if(e.event&Event::CHARACTER_MOVE_UP) fprintf(stdout, "[CHARACTER_MOVE_UP]");
	if(e.event&Event::CHARACTER_MOVE_DOWN) fprintf(stdout, "[CHARACTER_MOVE_DOWN]");
	if(e.event&Event::CHARACTER_CHARGE) fprintf(stdout, "[CHARACTER_CHARGE]");
	if(e.event&Event::CHARACTER_SAY) fprintf(stdout, "[CHARACTER_SAY]");
	if(e.event&Event::CHARACTER_DEFEND) fprintf(stdout, "[CHARACTER_DEFEND]");
	if(e.event&Event::CHARACTER_HIT) fprintf(stdout, "[CHARACTER_HIT]");
	if(e.event&Event::CHARACTER_HIT_SELF) fprintf(stdout, "[CHARACTER_HIT_SELF]");
	if(e.event&Event::CHARACTER_JUMP) fprintf(stdout, "[CHARACTER_JUMP]");
	if(e.event&Event::CHARACTER_COMBO) fprintf(stdout, "[CHARACTER_COMBO]");
	if(e.event&Event::CHARACTER_SOUND) fprintf(stdout, "[CHARACTER_SOUND]");
	if(e.event&Event::CHARACTER_MISC) fprintf(stdout, "[CHARACTER_MISC]");
	fprintf(stdout, "; ");
	if(c->getState()&Character::CONTROLLABLE) fprintf(stdout, "[CONTROLLABLE]");
	if(c->getState()&Character::MOVING) fprintf(stdout, "[MOVING]");
	if(c->getState()&Character::DEFENDING) fprintf(stdout, "[DEFENDING]");
	if(c->getState()&Character::CHARGING) fprintf(stdout, "[CHARGING]");
	if(c->getState()&Character::SPEAKING) fprintf(stdout, "[SPEAKING]");
	if(c->getState()&Character::COMBO) fprintf(stdout, "[COMBO]");
	if(c->getState()&Character::DEAD) fprintf(stdout, "[DEAD]");
	if(c->getState()&Character::DISABLED) fprintf(stdout, "[DISABLED]");
	fprintf(stdout, "\n");
	fflush(stdout);*/

	broadcast(EVENT_CHARACTER, sizeof(ec), (char*)&ec);
	}

void Server::sendRequestCharacter(int socket, unsigned crc)
	{
	ServerPlayer *sp=findPlayerBySocketID(socket);

	if(!sp)
		{
		Log::fail("Server.sendRequestCharacter: Niewiele brakowalo a wyslanoby request postaci do nieistniejacego gracza (socket %d)", socket);
		return;
		}

	sp->transfer.download(Request::CHARACTER, crc);
	}


