/*
 * buffer.cpp
 *
 *  Created on: 11-02-2013
 *      Author: crm
 */

#include "buffer.h"

#include <GL/glee.h>

#include "asmmath.h"
#include "log.h"

#include "graphics/image.h"
#include "graphics/sprite.h"
#include "render.h"
#include "orientation.h"

bool Render::Buffer::init()
	{
	if(initd)
		clear();

	/*if(glGenBuffers)	// <- wut?
		glGenBuffers(1, &id);

	int err=glGetError();

	if(!id || err!=GL_NO_ERROR)
		{
		Log::fail("Buffer.init: Nie udalo sie dostac bufora (%d, %d)", id, err);

		initd=false;
		return false;
		}*/

	initd=true;
	return true;
	}

void Render::Buffer::clear()
	{
	flush();

	initd=false;

	if(id)
		glDeleteBuffers(1, &id);
	id=0u;
	}

void Render::Buffer::flush()
	{
	verts.clear();

	count=0;
	}

/**** Buforek 2D ****/
void Render::Buffer2D::clear()
	{
	spr.clear();

	Buffer::clear();
	}

void Render::Buffer2D::bind() const
	{
	//Render::bind(spr);	// wtf. Operator bool gdy funkcja chce uinta?
	Render::bind(spr->getImg()->tex[0]);
	}

void Render::Buffer2D::setSpr(Graphics::SpritePtr& spr)
	{
	this->spr=spr;
	}

void Render::Buffer2D::print(const Orientation& orientation, float x, float y, float w, float h, float alpha)
	{
	const float iw=spr->getImg()->w;
	const float ih=spr->getImg()->h;
	const AVector s=orientation.getScale();
	const AVector p=orientation.getPos();

	const AVector tl=p+orientation.getForward()*h*(s.y-1.0f);
	const AVector tr=p+orientation.getForward()*h*(s.y-1.0f)+orientation.getRight()*w;
	const AVector bl=p-orientation.getForward()*h;
	const AVector br=p-orientation.getForward()*h+orientation.getRight()*w;

	verts.push_back( Vertex(tl.x, tl.y, tl.z,  x   /iw,  y   /ih, 255.0f*alpha, 255u, 255u, 255u) );
	verts.push_back( Vertex(tr.x, tr.y, tr.z, (x+w)/iw,  y   /ih, 255.0f*alpha, 255u, 255u, 255u) );
	verts.push_back( Vertex(br.x, br.y, br.z, (x+w)/iw, (y+h)/ih, 255.0f*alpha, 255u, 255u, 255u) );
	verts.push_back( Vertex(bl.x, bl.y, bl.z,  x   /iw, (y+h)/ih, 255.0f*alpha, 255u, 255u, 255u) );

	count+=4;
	}

void Render::Buffer2D::print(const Orientation& orientation, float alpha)
	{
	const Graphics::Image* img=spr->getImg();

	print(orientation, 0, 0, img->w, img->h, alpha);
	}

void Render::Buffer2D::print(const Orientation& orientation, unsigned animation, unsigned direction, unsigned frame, float alpha)
	{
	const Graphics::Animation& a=spr->getAnim(animation);
	const Graphics::Direction& d=a.getDir((Graphics::Directions)direction);
	const Graphics::Frame& f=d.getFrame(frame);

	const AVector pos=orientation.getPos()+orientation.getForward()*f.spoty-orientation.getRight()*f.spotx;

	print(Orientation(pos, orientation), f.x, f.y, f.w, f.h, alpha);
	}



