/*
 * sprite.h
 *
 *  Created on: 22-02-2013
 *      Author: crm
 */

#pragma once

#include <vector>
#include <string>
#include <map>

#include "refcounter.h"

namespace Graphics
	{
	struct Image;

	enum Directions
		{
		RIGHT=0,
		RIGHT_DOWN=1,
		DOWN=2,
		LEFT_DOWN=3,
		LEFT=4,
		LEFT_UP=5,
		UP=6,
		RIGHT_UP=7,

		DEFAULT=DOWN,
		COUNT=8,
		};

	struct Frame
		{
		Frame(int x, int y, int w, int h, int spotx=0, int spoty=0, int actx=0, int acty=0):
			x(x), y(y), w(w), h(h), spotx(spotx), spoty(spoty), actx(actx), acty(acty)
			{}

		int x, y, w, h, spotx, spoty, actx, acty;
		};

	class Direction
		{
		private:
			float speed;
			unsigned ret;

			std::vector<Frame> frames;

		public:
			 Direction();
			~Direction();

			void clear();

			inline float getSpeed() const {return speed;}
			inline unsigned getReturn() const {return ret;}
			inline unsigned getFrames() const {return frames.size();}
			inline const Frame& getFrame(unsigned i) const {return frames[i];}

			inline void setSpeed(float s) {speed=s;}
			inline void setReturn(unsigned s) {ret=s;}

			inline void addFrame(const Frame& f) {frames.push_back(f);}
		};

	class Animation
		{
		private:
			Direction *dirs[COUNT];

		public:
			 Animation(bool noinit=false);
			~Animation();

			void init();	// inicjalizacja domyslnego kierunku, wywolywane z konstruktora
			void clear();

			inline const Direction& getDir(Directions dir) const {return *dirs[dir];}

			//inline void addFrame(Directions dir, const Frame& f) {dirs[dir]->addFrame(f);}
			void addDirection(Directions dir, Direction *pdir);
		};

	class Sprite: public RefCounter
		{
		private:
			const Image *img;

			const std::string name;

			std::vector<Animation *> anims;
			std::map<std::string, Animation *> named;

		public:
			Sprite(const std::string& path="");
			virtual ~Sprite();

			void clear();

			inline const Image* getImg() const {return img;}
			inline const Animation& getAnim(unsigned i) const {return *anims[i];}
			inline const Animation& getAnim(const std::string& name) const {return *named.find(name)->second;}
			inline unsigned getAnims() const {return anims.size();}
			inline const std::string getName() const {return name;}

			unsigned getAnimID(const Animation *a) const;
			unsigned getAnimID(const std::string& name) const;

			void setImage(const Image *img);

			unsigned addAnimation(Animation *ani, const std::string& name="") {setAnimation(anims.size(), ani, name); return anims.size()-1;}
			void setAnimation(unsigned i, Animation *ani, const std::string& name="");
		};
	}
