/*
 * spriteptr.cpp
 *
 *  Created on: 23-02-2013
 *      Author: crm
 */

#include "spriteptr.h"

#include "sprite.h"
#include "spritemanager.h"

#include <cstdio>

namespace Graphics
	{
	SpritePtr::SpritePtr():
		spr(NULL), animID(0u), dirID(DEFAULT), anim(0u), dir(0u), frame(0.0f), alpha(1.0f), scale(1.0f), angle(0.0f)
		{
		setSprite(NULL);
		}

	SpritePtr::SpritePtr(const Sprite *s):
		spr(NULL), animID(0u), dirID(DEFAULT), anim(0u), dir(0u), frame(0.0f), alpha(1.0f), scale(1.0f), angle(0.0f)
		{
		setSprite(s);
		}

	SpritePtr::SpritePtr(const SpritePtr& p):
		spr(NULL), animID(0u), dirID(DEFAULT), anim(0u), dir(0u), frame(0.0f), alpha(1.0f), scale(1.0f), angle(0.0f)
		{
		setSprite(p.spr);
		}

	SpritePtr::SpritePtr(const std::string& name):
		spr(NULL), animID(0u), dirID(DEFAULT), anim(0u), dir(0u), frame(0.0f), alpha(1.0f), scale(1.0f), angle(0.0f)
		{
		setSprite(name);
		}

	SpritePtr::~SpritePtr()
		{
		clear();
		}


	void SpritePtr::clear()
		{
		setSprite(NULL);
		}

	void SpritePtr::update(float dt)
		{
		//spr->animate(anim, (Directions)dir, frame);

		frame+=dir->getSpeed()*dt;

		if(frame>=dir->getFrames())
			frame-=dir->getFrames();
		}


	bool SpritePtr::loop() const
		{
		return dir->getSpeed()==0.0f || dir->getReturn()<dir->getFrames()-1u;
		}

	bool SpritePtr::ended() const
		{
		return loop() || frame>=dir->getReturn();
		}


	void SpritePtr::setSprite(const Sprite *s)
		{
		if(spr)
			{
			spr->unref();
			if(spr->refs()<=0)
				SpriteManager::free(spr);
			}
		spr=s;
		if(spr)
			{
			spr->ref();
			animID=-1;

			// todo sprawdzic czy jest w ogole jakas animacja/klatka? Czy moze zakladam, ze sprite zawsze sa dobre?

			setAnim(0);
			}
		}

	void SpritePtr::setSprite(const SpritePtr& p)
		{
		setSprite(p.spr);
		}

	void SpritePtr::setSprite(const std::string& name)
		{
		setSprite(SpriteManager::get(name));
		}


	void SpritePtr::setAnim(unsigned sa)
		{
		if(animID==sa)
			return;

		anim=&spr->getAnim(sa);
		animID=sa;
		//dir=&anim->getDir(DEFAULT);

		frame=0.0f;

		unsigned tmp=dirID;
		dirID=-1;
		setDir(tmp);
		}

	void SpritePtr::setAnim(const std::string& sa)
		{
		if(animID==spr->getAnimID(sa))
			return;

		anim=&spr->getAnim(sa);
		animID=spr->getAnimID(anim);
		//dir=&anim->getDir(DEFAULT);

		unsigned tmp=dirID;
		dirID=-1;
		setDir(tmp);
		}

	void SpritePtr::setDir(int sd)
		{
		if(dirID==(unsigned)sd)
			return;
		if(sd>=COUNT)
			sd=sd%COUNT;

		dir=&anim->getDir((Directions)sd);
		dirID=sd;

		frame=0.0f;
		}


	void SpritePtr::getSize(int& gw, int& gh) const
		{
		const Frame& f=dir->getFrame(frame);

		gw=f.w;
		gh=f.h;
		}

	int SpritePtr::getW() const
		{
		return dir->getFrame(frame).w;
		}

	int SpritePtr::getH() const
		{
		return dir->getFrame(frame).h;
		}

	void SpritePtr::getHotspot(int& gx, int& gy) const
		{
		const Frame& f=dir->getFrame(frame);

		gx=f.spotx;
		gx=f.spoty;
		}

	int SpritePtr::getHotspotX() const
		{
		return dir->getFrame(frame).spotx;
		}

	int SpritePtr::getHotspotY() const
		{
		return dir->getFrame(frame).spoty;
		}

	void SpritePtr::getActionPoint(int& gx, int& gy) const
		{
		const Frame& f=dir->getFrame(frame);

		gx=f.actx;
		gy=f.acty;
		}

	int SpritePtr::getActionPointX() const
		{
		return dir->getFrame(frame).actx;
		}

	int SpritePtr::getActionPointY() const
		{
		return dir->getFrame(frame).acty;
		}

	unsigned SpritePtr::getAnimationCount() const
		{
		return spr->getAnims();
		}

	unsigned SpritePtr::getFrameCount() const
		{
		return dir->getFrames();
		}
	}
