/*
 * main.cpp
 *
 *  Created on: 22-02-2013
 *      Author: crm
 */

#include <ctime>
#include <cstdlib>
#include <SDL/SDL.h>

#include "resource.h"
#include "log.h"

#include "consts.h"
#include "engine.h"
#include "render.h"
#include "sound.h"
#include "graphics/imagemanager.h"
#include "graphics/spritemanager.h"

#include "camera.h"
#include "graphics/image.h"
#include "graphics/sprite.h"
#include "ui/text.h"

#include <cmath>

SDL_Event e;

Uint8 *key=SDL_GetKeyState(NULL);

bool run=		true;
float delta=	0.0f;

#undef main

/*int main(int argc, char *argv[])
	{
	AVector a;
	Orientation o;

	printf("start");

	printf("o r %f %f %f", o.getRight().x, o.getRight().y, o.getRight().z);
	printf("o f %f %f %f", o.getForward().x, o.getForward().y, o.getForward().z);
	printf("o u %f %f %f", o.getUp().x, o.getUp().y, o.getUp().z);
	printf("o r %.02f %.02f %.02f; f %.02f %.02f %.02f; u %.02f %.02f %.02f", o.getRight().x, o.getRight().y, o.getRight().z, o.getForward().x, o.getForward().y, o.getForward().z, o.getUp().x, o.getUp().y, o.getUp().z);
	o.rotateUp(90.0f);
	printf("o r %.02f %.02f %.02f; f %.02f %.02f %.02f; u %.02f %.02f %.02f", o.getRight().x, o.getRight().y, o.getRight().z, o.getForward().x, o.getForward().y, o.getForward().z, o.getUp().x, o.getUp().y, o.getUp().z);
	o.rotateRight(90.0f);
	printf("o r %.02f %.02f %.02f; f %.02f %.02f %.02f; u %.02f %.02f %.02f", o.getRight().x, o.getRight().y, o.getRight().z, o.getForward().x, o.getForward().y, o.getForward().z, o.getUp().x, o.getUp().y, o.getUp().z);
	o.rotateForward(90.0f);
	printf("o r %.02f %.02f %.02f; f %.02f %.02f %.02f; u %.02f %.02f %.02f", o.getRight().x, o.getRight().y, o.getRight().z, o.getForward().x, o.getForward().y, o.getForward().z, o.getUp().x, o.getUp().y, o.getUp().z);

	a=AVector(1, 0, 0);
	printf("a %f %f %f", a.x, a.y, a.z);
	a=o.getMatrix()*a;
	printf("a %f %f %f", a.x, a.y, a.z);

	a=AVector(1, 1, 0);
	a=AMatrixTranspose(o.getMatrix())*a;
	printf("a %f %f %f", a.x, a.y, a.z);

	return 0;
	}*/

int main(int argc, char *argv[])
	{
#ifdef RELEASE
	Log::setOut(fopen("log.txt", "w+"));
	Log::setLogLvl(Log::INFO|Log::FAIL);
#else
	Log::setLogLvl(Log::ALL|Log::DEBUG);
#endif

	Log::info("Start");
	Log::info("Build %s %s", __DATE__, __TIME__);

		{
		int s=time(NULL);
		srand(s);

		Log::debug("Seed: 0x%08X", s);
		}

	{
	Orientation o(Orientation::FLAT_XY);
	//o.rotateZ(45.0f);
	o.move(AVector(1, 0, 0));
	AVector v(1, 0, 0, 1);

	AMatrix m=o.getMatrix();

	for(int i=0; i<4; ++i)
		printf("m[%d] %f %f %f %f", i, m.row[i].x, m.row[i].y, m.row[i].z, m.row[i].w);
	printf("");
	printf("v    %f %f %f %f", v.x, v.y, v.z, v.w);
	printf("");

	AVector out=m*v;

	printf("out  %f %f %f\n", out.x, out.y, out.z);
	}

	//return 0;

	if(!Resource::includeList(RESLIST))
		return __LINE__;

	if(!Engine::init())
		return __LINE__;

	if(!Render::init(SCREENWIDTH, SCREENHEIGHT, GAMENAME, ""))
		return __LINE__;

	// Loading~
		{
		Render::Camera cam(Render::Camera::GUI, 0, 0, Render::getWindowSize().x, Render::getWindowSize().y, -1, 1, 0.0f, 1.0f, false);
		cam.look(AVector(0, 0, 0), AVector(0, 0, 1), AVector(0, -1, 0));
		Render::setCamera(&cam);
		UI::Text loading(DEFAULT_FONT, "Wczytywanie...", Render::getWindowSize().x, Render::getWindowSize().y);
		loading.setAlignCenter();
		loading.setAlignMiddle();
		loading.update();
		loading.print(Orientation(AVector(0, Render::getWindowSize().y, 0)), 1.0f);
		Render::update();

		SDL_Delay(100);
		}

	if(!Sound::init(32))
		return __LINE__;

	/**** Do pozniejszego wywalenia~? ****/
#ifndef RELEASE
	float lfps=0.0f;
	float ltime=0.0f;

	UI::Text info(DEFAULT_FONT, "Plx w8", Render::getWindowSize().x, 0);
	info.update();

	UI::Text gps(DEFAULT_FONT, "-|-", Render::getWindowSize().x, 0);
	gps.setAlignCenter();

	unsigned fpsTimer=	SDL_GetTicks();
	unsigned fpsFrames=	0u;
#endif
	/****/
	Log::info("main: init %ums", SDL_GetTicks());

	Render::Camera cam;
	cam=Render::Camera(Render::Camera::PERSPECTIVE, 0, 0, Render::getWindowSize().x, Render::getWindowSize().y, 1, SCREENFAR, 60.0f, 1.0f, true);
	//cam.look(AVectorZero(), AVector(-128, -128, 64), AVector(0, 0, 1));
	//cam.look(AVector(0, 0, 0), AVector(0, -128, 64), AVector(0, 0, 1));
	cam.setPos(AVector(0, -128, 48));
	/*cam.look(AVectorZero(), 128, 0.0f, 0.0f);

	float ang=0.0f;
	float elev=0.0f;*/

	Render::Camera camui;
	camui=Render::Camera(Render::Camera::GUI, 0, 0, Render::getWindowSize().x, Render::getWindowSize().y, -100, 100, 0.0f, 1.0f, false);
	//camui.look(AVector(0, 0, 0), AVector(0, 0, 1), AVector(0, -1, 0));

	Render::setCamera(&cam);

	Graphics::SpritePtr spr("npc/mc.xml");
	//spr.setAnim("spin");

	Graphics::SpritePtr spr2("npc/dirtest.xml");

	Graphics::SpritePtr icon("icon.xml");
	icon.setAnim("test");

	Graphics::SpritePtr grass;
	grass.setSprite("img/screen.png");

	const Graphics::Image *img=Graphics::ImageManager::get("img/screen.png");

	Log::info("------------------------------------------------------------");

	float fov=60;

	char infobuff[128];
	unsigned frameTime;
	while(run)
		{
		spr.update(delta);

		frameTime=SDL_GetTicks();
		while(SDL_PollEvent(&e))
			{
			switch(e.type)
				{
				case SDL_QUIT:
					run=false;
				break;

				case SDL_KEYDOWN:
					if( (e.key.keysym.sym==SDLK_F4 && (e.key.keysym.mod&KMOD_ALT)) ||
						 e.key.keysym.sym==SDLK_ESCAPE)
						run=false;

					if(e.key.keysym.sym==SDLK_z)
						spr2.setDir(spr2.getDirID()+1);

				break;

				case SDL_MOUSEMOTION:
					if(!(SDL_GetMouseState(NULL, NULL)&SDL_BUTTON(3)))
						break;
					cam.rotateZ(-e.motion.xrel*delta*10);
					cam.rotateRight(-e.motion.yrel*delta*10);
				break;

				default:
					//
				break;
				}
			}

		if(SDL_GetMouseState(NULL, NULL)&SDL_BUTTON(3))
			{
			SDL_EventState(SDL_MOUSEMOTION, SDL_DISABLE);
			SDL_WarpMouse(SCREENWIDTH/2, SCREENHEIGHT/2);
			SDL_EventState(SDL_MOUSEMOTION, SDL_ENABLE);
			}

	/**** Update ****/
		if(key[SDLK_w]) cam.moveForward( (128+512*key[SDLK_LSHIFT])*delta);
		if(key[SDLK_s]) cam.moveForward(-(128+512*key[SDLK_LSHIFT])*delta);
		if(key[SDLK_a]) cam.moveRight(-(128+512*key[SDLK_LSHIFT])*delta);
		if(key[SDLK_d]) cam.moveRight( (128+512*key[SDLK_LSHIFT])*delta);
		//if(key[SDLK_SPACE]) cam.move(AVector(0, 0,  (128+512*key[SDLK_LSHIFT]))*delta);
		//if(key[SDLK_LCTRL]) cam.move(AVector(0, 0, -(128+512*key[SDLK_LSHIFT]))*delta);
		if(key[SDLK_SPACE]) cam.moveUp( (128+512*key[SDLK_LSHIFT])*delta);
		if(key[SDLK_LCTRL]) cam.moveUp(-(128+512*key[SDLK_LSHIFT])*delta);
		if(key[SDLK_q]) cam.rotateForward( 45*delta);
		if(key[SDLK_e]) cam.rotateForward(-45*delta);
		if(key[SDLK_1]) cam.setType(cam.ORTHOGRAPHIC);
		if(key[SDLK_2]) cam.setType(cam.PERSPECTIVE);
		if(key[SDLK_KP_PLUS] && fov<170) {fov+=20*delta; cam.setFov(fov);}
		if(key[SDLK_KP_MINUS] && fov>10) {fov-=20*delta; cam.setFov(fov);}

		AVector scrpos=cam.getPointOnScreen(AVector(0, 0, 0));

		printf("%f %f", scrpos.x, scrpos.y);

		/*Render::setCamera(Render::CAMERA_UI);
		Render::drawLine(AVector(0, 0), scrpos);
		Render::setCamera(&cam);*/

		/*if(key[SDLK_z])  ang-=45*delta;
		if(key[SDLK_x])  ang+=45*delta;
		if(key[SDLK_c]) elev-=45*delta;
		if(key[SDLK_v]) elev+=45*delta;*/

		//cam.look(AVector(0, 64, 0), AVector(cos(SDL_GetTicks()/2000.0f*M_PI)*128, sin(SDL_GetTicks()/2000.0f*M_PI)*128+64, 96), AVector(0, 0, 1));
		//cam.look(AVectorZero(), 128, elev, ang);

		Render::setCamera(&cam);
		Render::setColor(1.0f, 1, 0, 0); Render::drawLine(AVector(0, 0, 0), cam.getPos()+cam.getForward()*1);

		Render::setColor(0.5f, 1, 1, 1);
		const int GRIDW=30;
		const int GRIDH=30;
		for(int x=0; x<=GRIDW; ++x)
			for(int y=0; y<=GRIDH; ++y)
				{
				Render::drawLine(AVector(-GRIDW/2*32+32*x, -GRIDW/2*32, 0), AVector(-GRIDW/2*32+32*x, GRIDW/2*32, 0));
				Render::drawLine(AVector(-GRIDW/2*32, -GRIDW/2*32+32*y, 0), AVector(GRIDW/2*32,  -GRIDW/2*32+32*y, 0));
				}

		Render::setColor(1.0f, 1.0f, 0, 0); Render::drawLine(AVector(0, 0, 2), AVector(32, 0,  2)); // r
		Render::setColor(1.0f, 0, 1.0f, 0); Render::drawLine(AVector(0, 0, 2), AVector(0, 32,  2)); // g
		Render::setColor(1.0f, 0, 0, 1.0f); Render::drawLine(AVector(0, 0, 2), AVector(0,  0, 34)); // b

		const AVector offset=cam.getPos()+cam.getForward()*8-cam.getUp()*2+cam.getRight()*2;
		Render::setColor(1.0f, 1.0f, 0, 0); Render::drawLine(offset, offset+cam.getRight()*8); // r
		Render::setColor(1.0f, 0, 1.0f, 0); Render::drawLine(offset, offset+cam.getForward()*8); // g
		Render::setColor(1.0f, 0, 0, 1.0f); Render::drawLine(offset, offset+cam.getUp()*8); // b
		Render::setColor(1, 1, 1, 1);
		//Render::print(img, Orientation(AVector(0, 0, 1), Orientation::FLAT_XY));
		Render::print(img, Orientation(AVector(256, 0, img->h), Orientation::FLAT_XZ));
		Render::print(img, Orientation(AVector(256, 0, img->h), Orientation::FLAT_YZ));
		Render::print(grass->getImg(), Orientation(AVector(-256, 256, 1), Orientation::FLAT_XY), 0, 0, 512, 512);

		Render::print(spr, Orientation(AVector(0, 0, 0), Orientation::FLAT_XZ));
		Render::print(spr, Orientation(AVector(32, 32, 0), cam.getBillboard(AVector(32, 32, 0))));
		Render::print(spr2, Orientation(AVector(0, 64, 0), cam.getBillboard(AVector(0, 64, 0))), 1.0f, true);
		const AVector pp=AVector(cos(SDL_GetTicks()/1000.0f*M_PI)*32+128, sin(SDL_GetTicks()/1000.0f*M_PI)*32+128, 0);
		Render::print(spr, Orientation(pp, cam.getBillboard(pp)));

		Render::print(icon, Orientation(AVector(64, 0, 128), cam.getUp(), -cam.getForward()));
		Render::print(icon, Orientation(AVector(64, 0, 140), cam.getUp(), -cam.getForward()));
		Render::setColor(1, 1, 0, 0);
		Render::drawLine(AVector(64, 0, 0), AVector(64, 0, 160));
		Render::setColor(1, 1, 1, 1);

		srand(1337);
		for(int i=0; i<80; ++i)
			{
			const AVector pos=AVector(-256, 128) + AVector(rand()/(RAND_MAX+1.0f)*1600, rand()/(RAND_MAX+1.0f)*1600, 0.0f);
			Render::print(spr2, Orientation(pos, cam.getBillboard(pos)), 1.0f, true);
			//Render::print(spr2, Orientation(pos, i%2?Orientation::FLAT_XZ:Orientation::FLAT_YZ), 1.0f);
			}


		Render::setCamera(&camui);
		Render::setColor(1, 1, 0, 0);
		Render::drawLine(AVector(8, 8, 8), AVector(8, 64, 8));
		Render::setColor(1, 0, 1, 0);
		Render::drawLine(AVector(8, 8, 8), AVector(64, 8, 8));
		Render::setColor(1, 0, 0, 1);
		Render::drawLine(AVector(8, 8, 8), AVector(8, 8, 64));
		Render::setColor(1, 0, 0, 0);
		Render::drawLine(AVector(8, -8, 0), AVector(8, -64, 0));
		Render::drawLine(AVector(8, -8, 0), AVector(64, -8, 0));
		Render::setColor(1.0f, 1.0f, 0.0f, 0.0f);
		Render::drawTriangle(AVector(8, 8, -1), AVector(20, 24, -1), AVector(32, 8, -1));

		Render::setColor(1, 1, 1, 1);
		Render::print(spr, Orientation(AVector(32, 32, 0)));
		Render::print(spr, Orientation(scrpos));

	/**** Debugowy overlay ****/
		info.print(Orientation(AVector(8, Render::getWindowSize().y-8, 0)), 0.5f);

		sprintf(infobuff, "%.02f, %.02f, %.02f", cam.getPos().x, cam.getPos().y, cam.getPos().z);
		gps=infobuff;
		gps.update();
		gps.print(Orientation(AVector(0, 20, 0)), 1.0f);

	/**** Reszta ****/
		Render::update();

	/**** Delta i inne framerate'y ****/
		delta=(SDL_GetTicks()-frameTime)/1000.0f;
		if(delta<DELTA_MIN)
			{
			SDL_Delay(1);
			delta=(SDL_GetTicks()-frameTime)/1000.0f;
			}
		else if(delta>DELTA_MAX)
			delta=DELTA_MAX;

	// Przeliczanie fps'ow
#ifndef RELEASE
		++fpsFrames;
		if(SDL_GetTicks()>fpsTimer+1000u)
			{
			sprintf(infobuff, "FPS: %.02f;\ttime: %.02f", lfps, ltime);
			info.setStr(infobuff);
			info.update();
			//Log::info("%s", infobuff);

			lfps=fpsFrames*1000.0f/(SDL_GetTicks()-fpsTimer);
			ltime=(SDL_GetTicks()-fpsTimer)/fpsFrames;

			fpsTimer=SDL_GetTicks();
			fpsFrames=0u;
			}
#endif
		}

	Log::info("------------------------------------------------------------");
	Log::info("Zatrzymywanie...");

	Graphics::SpriteManager::clear();
	Graphics::ImageManager::clear();
	Render::clear();
	Resource::clear();

	Log::info("Stop");
	Log::close();

	return 0;
	}


