/*
 * orientation.h
 *
 *  Created on: 26-05-2014
 *      Author: crm
 */

#pragma once

#include "asmmath.h"

//namespace Engine
//	{
	class Orientation
		{
		public:
			//

		protected:
			AVector pos;

			AVector right;
			AVector forward;
			AVector up;
			AVector scale;

		public:
			Orientation():
				pos(), right(1, 0, 0), forward(0, 1, 0), up(0, 0, 1), scale(1, 1, 1) {this->pos.w=1;}
			Orientation(const Orientation& o):
				pos(o.pos), right(o.right), forward(o.forward), up(o.up), scale(o.scale) {this->pos.w=1;}
			Orientation(const AVector& pos, const AVector& scale=AVector(1, 1, 1)):
				pos(pos), right(1, 0, 0), forward(0, 1, 0), up(0, 0, 1), scale(scale) {this->pos.w=1;}
			Orientation(const AVector& pos, const Orientation& o, const AVector& scale=AVector(1, 1, 1)):
				pos(pos), right(o.right), forward(o.forward), up(o.up), scale(o.scale*scale) {this->pos.w=1;}
			Orientation(const AVector& pos, const AVector& forward, const AVector& up, const AVector& scale=AVector(1, 1, 1)):
				pos(pos), right(forward^up), forward(forward), up(up), scale(scale) {this->pos.w=1;}
			Orientation(const AVector& pos, float ang, float pitch, float roll, const AVector& scale=AVector(1, 1, 1)):
				pos(pos), right(1, 0, 0), forward(0, 1, 0), up(0, 0, 1), scale(scale)
				{
				this->pos.w=1;
				rotateUp(ang);
				rotateRight(pitch);
				rotateForward(roll);
				}
			virtual ~Orientation() {}

			AVector getPos() const {return pos;}
			AVector getRight() const {return right;}
			AVector getForward() const {return forward;}
			AVector getUp() const {return up;}
			AVector getScale() const {return scale;}

			Orientation& setPos(const AVector& v) {pos=v; return *this;}
			Orientation& setScale(const AVector& s) {scale=s; return *this;}

			void move(const AVector& v) {pos=pos+v;}
			void moveRight(float dist) {pos=pos+right*dist;}
			void moveForward(float dist) {pos=pos+forward*dist;}
			void moveUp(float dist) {pos=pos+up*dist;}
			void rotate(float deg, const AVector& axis=AVector(0, 0, 1));
			void rotateRight(float deg);
			void rotateForward(float deg);
			void rotateUp(float deg);
			void rotateX(float deg);
			void rotateY(float deg);
			void rotateZ(float deg);

			void look(const AVector& target, const AVector& pos, const AVector& up);
			void look(const AVector& target, float dist, float elev, float deg=0.0f);	// elev -> obrot wokol right, angle -> obrot wokol Z

			//AMatrix getMatrix() const {return AMatrixTranspose(AMatrix(right, -up, -forward, AVector(0, 0, 0, 1)));}
			AMatrix getMatrix() const;// {return AMatrixTranspose(AMatrix(right, forward, up, AVector(-pos.x, -pos.y, -pos.z, 1)));}

		public:
			static const Orientation FLAT_XY;
			static const Orientation FLAT_XZ;
			static const Orientation FLAT_YZ;
			static const Orientation GUI;
		};
//	}
