/*
 * Render.h
 *
 *  Created on: 30-07-2012
 *      Author: crm
 */

#pragma once

#include <vector>
#include <string>

#include "asmmath.h"

#include "orientation.h"

// Forwardy
namespace Graphics
	{
	struct Image;
	class Sprite;
	class SpritePtr;
	}

namespace Render
	{
	class Camera;
	class Buffer;
	}

// Tutejsze
namespace Render
	{
	enum CameraID
		{
		CAMERA_DEFAULT= 0,
		CAMERA_UI=		1,

		CAMERA_COUNT
		};

	/**** Funkcje~ ****/
	bool init(int width, int height, const std::string& title=std::string(), const std::string& icon=std::string());
	void update();						// Wyplucie wszystkiego na ekran?
	void clear();						// Skasowanie wszystkich grafik, buforow, zniszczenie okienka?

	void bind(unsigned tex);
	void bind(const Graphics::Image *img);
	void bind(const Graphics::Sprite *spr);
	void bind(const Graphics::SpritePtr& spr);

	void setColor(float a, float r=1.0f, float g=1.0f, float b=1.0f);
	inline void setAlpha(float a) {setColor(a);}

	void setCamera(Camera *cam);
	void setCamera(CameraID cam);
	Camera* getCamera();

	bool getFullscreen();
	AVector getWindowSize();
	void setFullscreen(bool state);

	void drawLine(const AVector& a, const AVector& b);
	void drawTriangle(const AVector& a, const AVector& b, const AVector& c);
	void drawRect(const AVector& a, const AVector& b, const AVector& c, const AVector& d);
	void drawTexturedTriangle(const AVector& a, const AVector& b, const AVector& c, float ax, float ay, float bx, float by, float cx, float cy);
	void drawTexturedRect(const AVector& a, const AVector& b, const AVector& c, const AVector& d, float ax, float ay, float bx, float by, float cx, float cy, float dx, float dy);

	void print(const Graphics::Image *img, const Orientation& orientation, float x, float y, float w, float h, float alpha=1.0f);
	void print(const Graphics::Image *img, const Orientation& orientation, float alpha=1.0f);
	void print(const Graphics::Sprite *spr, const Orientation& orientation, unsigned animation, unsigned direction, unsigned frame, float alpha=1.0f);
	void print(const Graphics::SpritePtr& spr, const Orientation& orientation, float alpha=1.0f, bool rotate=false);
	void print(Render::Buffer& buff, const Orientation& orientation, float alpha=1.0f);
	}

