#include "textttf.h"

#include <cstring>
#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

#include "../graphics/sprite.h"
#include "../render.h"

#include "log.h"

namespace UI
	{
	bool TextTTF::init(const std::string& font, int size, int sw, int sh, const std::string& str, int sline)
		{
		w=sw;
		h=sh;

		line=sline;

		setFont(font, size);
		setStr(str);

		return true;
		}

	void TextTTF::update()
		{
		if(!upd)
			return;
		upd=false;

		parse();
		print();
		}

	void TextTTF::print(float x, float y, float z, float alpha)
		{
		Render::setAlpha(alpha*255);
		//Render::print(img, x, y, z, alpha);
		}

	void TextTTF::clear()
		{
		upd=true;

		if(font)
			TTF_CloseFont(font);
		if(srf)
			SDL_FreeSurface(srf);
		if(img)
			;//

		font=NULL;
		srf=NULL;
		img=NULL;
		}
	}

namespace UI
	{
	void TextTTF::setStr(const std::string& str)
		{
		upd=true;

		this->str=str;
		}

	void TextTTF::addStr(const std::string& astr)
		{
		setStr(str+astr);
		}

	void TextTTF::setFont(const std::string& name, int size)
		{
		font=TTF_OpenFont(name.c_str(), size);

		if(!font)
			return;

		if(line<0)
			line=TTF_FontHeight(font)+1;
		}
	}

namespace UI
	{
	void TextTTF::parse()
		{
		// Na poczatek - skasuj poprzednie informacje
		lines.clear();

		LineInfo li;
		li.start=0;
		li.length=0;
		li.pixs=0;

	// Przelot pierwszy, lamanie po enterach, dlugosc linii w znakach
		for(unsigned i=0u; i<str.size(); ++i)
			{
			char c=str[i];
		// Nowa linia
			if((c=='\r' && i+1<str.size() && str[i+1]=='\n') || c=='\n')
				{
				lines.push_back(li);
				li.start=i+(c=='\r')+1;
				li.length=0;
				//li.pixs=0;
				}
			else
				++li.length;
			}
		lines.push_back(li);

		//printf("Podzielone po enterach, linii %u", lines.size());

	// Przelot drugi, trimowanie i takie tam
		trim();

	// Przeliczenie dlugosci linii
		//for(unsigned l=0u; l<lines.size(); ++l)
		/*for(std::list<LineInfo>::iterator it=lines.begin(); it!=lines.end(); ++it)
			{
			LineInfo& li=*it;//lines[l];

			li.pixs=getLineLen(li.start, li.length);
			}*/

	// Wrapowanie, opcjonalne
		if(opts&WRAP)
			{
			wrap();
			trim();
			}

	// Rzeczywiste wymiary
		h=lines.size()*line;

		if(opts&LEFT)
			for(std::list<LineInfo>::iterator it=lines.begin(); it!=lines.end(); ++it)
				w=std::max(w, it->pixs);
		}

	void TextTTF::print()
		{
		const int RMASK=0xFF000000;
		const int GMASK=0x00FF0000;
		const int BMASK=0x0000FF00;
		const int AMASK=0x000000FF;

		int x;
		int y;

		std::string lstr;
		SDL_Surface *lsrf;
		SDL_Rect lpos;
		SDL_Color col;

		if(opts&TOP)			y=0;
		else if(opts&MIDDLE)	y=0;	// todo wyrownanie w pionie
		else if(opts&BOTTOM)	y=0;	// ^

		//printf("str: [%s]", str.c_str());

		col.r=0;
		col.g=127;
		col.b=255;


		srf=SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 32, RMASK, GMASK, BMASK, AMASK);
		memset(srf->pixels, 0x00, w*h*4);
		SDL_SetAlpha(srf, 0, 0);

		for(std::list<LineInfo>::iterator it=lines.begin(); it!=lines.end(); ++it)
			{
			LineInfo& li=*it;

			x=0;
			if(opts&LEFT)			x=0;
			else if(opts&CENTER)	x=(w-li.pixs)/2;
			else if(opts&RIGHT)		x= w-li.pixs;

			lstr=str.substr(li.start, li.length);
			lsrf=TTF_RenderUTF8_Solid(font, lstr.c_str(), col);
			lpos.x=x;
			lpos.y=y;

			if(SDL_UpperBlit(lsrf, NULL, srf, &lpos)<0)
				printf("fail >:");

			SDL_FreeSurface(lsrf);
			}

		SDL_SaveBMP(srf, "text.bmp");
		}
	}

namespace UI
	{
	void TextTTF::trim()
		{
		//for(unsigned l=0u; l<lines.size(); ++l)
		/*for(std::list<LineInfo>::iterator it=lines.begin(); it!=lines.end(); ++it)
			{
			LineInfo& li=*it;//lines[l];

			if(opts&(RIGHT|CENTER))
				{
				for(int i=li.start; i<li.start+li.length; ++i)
					{
					if((unsigned char)str[i]>' ')
						break;

					++li.start;
					--li.length;
					}
				}

			if(opts&(LEFT|CENTER))
				{
				for(int i=li.start+li.length-1; i>li.start; --i)
					{
					if((unsigned char)str[i]>' ')
						break;

					--li.length;
					}
				}
			}*/
		}

	void TextTTF::wrap()
		{
		//for(unsigned l=0u; l<lines.size(); ++l)
		/*for(std::list<LineInfo>::iterator it=lines.begin(); it!=lines.end(); ++it)
			{
			LineInfo& li=*it;//lines[l];

			int pixs=0;

			for(int i=li.start; i<li.start+li.length; ++i)
				{
				unsigned char c=str[i];

				pixs+=getCharLen(c, pixs);

				//if(c<=' ' && (pixs>w || pixs+getWordLen(i+1, li.length-i+li.start-1)>w))
				if(c<=' ' && pixs+getWordLen(i+1, li.length-i+li.start-1)>w)
					{
					LineInfo nl;

					nl.start=i;
					nl.length=li.length-i+li.start;

					li.length-=nl.length;

					for(int t=nl.start; t<nl.start+nl.length; ++t)
						if((unsigned char)str[t]<=' ')
							++nl.start,
							--nl.length;
						else
							break;

					it=lines.insert( ++it, nl );
					--it;
					break;
					}
				}
			}*/
		}
	}

namespace UI
	{
	int TextTTF::getSubstrLength(int start, int len)		// Zwraca dlugosc najblizszego slowa w pixelach
		{
		if(!font)
			return 0;

		int ww, wh;
		std::string wstr;

		wstr=str.substr(start, len);
		TTF_SizeText(font, wstr.c_str(), &ww, &wh);

		return ww;
		}
	}
