#pragma once

#include <string>
#include <list>

//#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

#include "../graphics/image.h"

//typedef TTF_Font;
struct SDL_Surface;

namespace UI
	{
	class TextTTF
		{
		public:
			enum Options
				{
			// Wyrownanie w poziomie
				LEFT=		0x1<<0,
				CENTER=		0x1<<1,
				RIGHT=		0x1<<2,
			// Wyrownanie w pionie
				TOP=		0x1<<3,
				MIDDLE=		0x1<<4,
				BOTTOM=		0x1<<5,

			// Zawijanie
				WRAP=		0x1<<6,					// Zawijanie tekstu

				DEFAULT=LEFT|TOP|WRAP,				// Domyslne wartosci
				};

		private:
			struct LineInfo
				{
				int start, length, pixs;			// poczatek, liczba znakow, dlugosc w pikselach
				};

			int getSubstrLength(int start, int length);

			void parse();							// Wypelnienie info o liniach
			void print();							// Wyrysowanie do bufora zgodnie z tym co w lines'ach

			void trim();
			void wrap();

			bool upd;
			float alpha;
			Options opts;

			TTF_Font *font;
			SDL_Surface *srf;
			Graphics::Image *img;

			std::string str;
			std::list<LineInfo> lines;

			int w, h;
			int line;

		public:
			TextTTF(): upd(false), alpha(0.0f), opts(DEFAULT), font(NULL), srf(NULL), img(NULL), str(), w(0), h(0), line(-1) {}
			virtual ~TextTTF() {clear();}

			TextTTF& operator= (const std::string& str) {setStr(str); return *this;}
			TextTTF& operator+=(const std::string& str) {addStr(str); return *this;}

			bool init(const std::string& font, int size, int w, int h, const std::string& str=std::string(), int line=-1);
			void update();
			void print(float x, float y, float z, float alpha=255.0f);
			void clear();

			void setStr(const std::string& str);
			void addStr(const std::string& str);

			void setFont(const std::string& font, int size=12);

			void setOptions(Options sopts) {opts=sopts; upd=true;}	// Nie jest to zbyt bezpieczne. Mozna zrobic rozne, dziwne kombinacje~
			void setAlignLeft()		{opts=(Options)((opts&~(CENTER|RIGHT))|LEFT); upd=true;}
			void setAlignCenter() 	{opts=(Options)((opts&~(LEFT|RIGHT))|CENTER); upd=true;}
			void setAlignRight()	{opts=(Options)((opts&~(LEFT|CENTER))|RIGHT); upd=true;}
			void setAlignTop()		{opts=(Options)((opts&~(MIDDLE|BOTTOM))|TOP); upd=true;}
			void setAlignMiddle() 	{opts=(Options)((opts&~(TOP|BOTTOM))|MIDDLE); upd=true;}
			void setAlignBottom()	{opts=(Options)((opts&~(TOP|MIDDLE))|BOTTOM); upd=true;}
			void setWrapOn()		{opts=(Options) (opts| WRAP); upd=true;}
			void setWrapOff()		{opts=(Options) (opts&~WRAP); upd=true;}
			void setW(int s)		{w=s; upd=true;}
			void setH(int s)		{h=s; upd=true;}
			void setDimension(int sw, int sh)	{w=sw; h=sh; upd=true;}
			void setAlpha(float sa)	{alpha=sa; print();}

			inline int getW() const {return w;}
			inline int getH() const {return h;}
			inline const std::string& getStr() const {return str;}
			inline int getOptions() const {return opts;}
			inline float getAlpha() const {return alpha;}
		};
	}
