/*
 * asmmath.h
 *
 *  Created on: 18-03-2012
 *      Author: crm
 */

#ifndef ASMMATH_H_
#define ASMMATH_H_

#ifdef M_PI
#undef M_PI
#endif

const double M_PI=3.14159265358979323846;

void nop();
void sincos(float val, float& s, float& c);

// Operacje wektorowe

struct AVector
	{
	explicit AVector(float x=0.0f, float y=0.0f, float z=0.0f, float w=0.0f):
		x(x), y(y), z(z), w(w) {}
	//AVector(float v): x(v), y(v), z(v), w(v) {}
	AVector(const AVector& a):
		x(a.x), y(a.y), z(a.z), w(a.w) {}

	AVector& operator=(const AVector& a) {x=a.x; y=a.y; z=a.z; w=a.w; return *this;}

	float x, y, z, w;
	};

struct AMatrix
	{
	explicit AMatrix(const AVector& a=AVector(), const AVector& b=AVector(), const AVector& c=AVector(), const AVector& d=AVector())
		{row[0]=a; row[1]=b; row[2]=c; row[3]=d;}
	AMatrix(const AMatrix& m) {row[0]=m.row[0]; row[1]=m.row[1]; row[2]=m.row[2]; row[3]=m.row[3];}

	void operator=(const AMatrix& m) {row[0]=m.row[0]; row[1]=m.row[1]; row[2]=m.row[2]; row[3]=m.row[3];}

	AVector row[4];
	};

AVector AVectorAdd(const AVector& a, const AVector& b);
AVector AVectorSubtract(const AVector& a, const AVector& b);
AVector AVectorMultiply(const AVector& a, const AVector& b);
AVector AVectorDivide(const AVector& a, const AVector& b);
//AVector AVectorShuffle(const AVector& a, char flag);
AVector AVectorNormalize(const AVector& a);
AVector AVectorLength(const AVector& a);

AVector AVectorDot(const AVector& a, const AVector& b);
AVector AVectorCross(const AVector& a, const AVector& b);

AVector AVectorAbsolute(const AVector& a);
AVector AVectorClamp(const AVector& a, const AVector& min, const AVector& max);

AVector AVectorZero();
AVector AVectorSet(float x, float y, float z, float w);
AVector AVectorReplicate(float val);

AVector AVectorTransform(const AVector& a, const AMatrix& m);
AVector AVectorMirror(const AVector& a, const AVector& mirror, const AVector& forward, const AVector& up=AVector(0, 0, 1));

AMatrix AMatrixTranspose(const AMatrix& m);
AMatrix AMatrixAdd(const AMatrix& a, const AMatrix& b);
AMatrix AMatrixSubtract(const AMatrix& a, const AMatrix& b);
AMatrix AMatrixMultiply(const AMatrix& a, const AMatrix& b);
AMatrix AMatrixInverse(const AMatrix& m);

AMatrix AMatrixAbsolute(const AMatrix& a);
AVector AMatrixDeterminant(const AMatrix& m);

AMatrix AMatrixRotX(float ang);
AMatrix AMatrixRotY(float ang);
AMatrix AMatrixRotZ(float ang);
AMatrix AMatrixRotate(float angle, const AVector& dir);
AMatrix AMatrixSkew(const AVector& a);

AMatrix AMatrixZero();
AMatrix AMatrixIdentity();
AMatrix AMatrixFill(float val);
AMatrix AMatrixDiagonal(float val);

// Odnosze dziwne wrazenie, ze operatorow moglo mi zawsze brakowac :F
// 20140211 20:11
inline AVector operator-(const AVector& a) {return AVector(-a.x, -a.y, -a.z, -a.w);}
inline AVector operator+(const AVector& a, float v) {return AVectorAdd(a, AVectorReplicate(v));}
inline AVector operator-(const AVector& a, float v) {return AVectorSubtract(a, AVectorReplicate(v));}
inline AVector operator*(const AVector& a, float v) {return AVectorMultiply(a, AVectorReplicate(v));}
inline AVector operator/(const AVector& a, float v) {return AVectorDivide(a, AVectorReplicate(v));}
inline AVector operator+(float v, const AVector& a) {return AVectorAdd(a, AVectorReplicate(v));}
inline AVector operator-(float v, const AVector& a) {return AVectorSubtract(a, AVectorReplicate(v));}
inline AVector operator*(float v, const AVector& a) {return AVectorMultiply(a, AVectorReplicate(v));}
inline AVector operator/(float v, const AVector& a) {return AVectorDivide(a, AVectorReplicate(v));}
inline AVector operator+(const AVector& a, const AVector& b) {return AVectorAdd(a, b);}
inline AVector operator-(const AVector& a, const AVector& b) {return AVectorSubtract(a, b);}
inline AVector operator*(const AVector& a, const AVector& b) {return AVectorMultiply(a, b);}
inline AVector operator/(const AVector& a, const AVector& b) {return AVectorDivide(a, b);}
inline float operator|(const AVector& a, const AVector& b) {return AVectorDot(a, b).w;}
inline AVector operator^(const AVector& a, const AVector& b) {return AVectorCross(a, b);}
inline AMatrix operator-(const AMatrix& a) {return AMatrix(-a.row[0], -a.row[1], -a.row[2], -a.row[3]);}
inline AMatrix operator+(const AMatrix& m, const AMatrix& n) {return AMatrixAdd(m, n);}
inline AMatrix operator-(const AMatrix& m, const AMatrix& n) {return AMatrixSubtract(m, n);}
inline AMatrix operator*(const AMatrix& m, const AMatrix& n) {return AMatrixMultiply(m, n);}
inline AVector operator*(const AMatrix& m, const AVector& v) {return AVectorTransform(v, m);}

#endif /* ASMMATH_H_ */
