/*
 * quaternion.h
 *
 *  Created on: 07-07-2014
 *      Author: crm
 */

#pragma once

#include "asmmath.h"

struct Quaternion
	{
	float x, y, z, w;

	explicit Quaternion(float x=1.0f, float y=0.0f, float z=0.0f, float w=0.0f): x(x), y(y), z(z), w(w) {}

	Quaternion operator=(const Quaternion& q) {x=q.x; y=q.y; z=q.z; w=q.w; return *this;}
	Quaternion operator+(const Quaternion& q) {return Quaternion(x+q.x, y+q.y, z+q.z, w+q.w);}
	Quaternion operator-(const Quaternion& q) {return Quaternion(x-q.x, y-q.y, z-q.z, w-q.w);}
	Quaternion operator*(const Quaternion& q) {return Quaternion(w*q.x+x*q.w+y*q.z-z*q.y,
															     w*q.y-x*q.z+y*q.w+z*q.x,
															     w*q.z+x*q.y-y*q.x+z*q.w,
															     w*q.w-x*q.x-y*q.y-z*q.z);}
	Quaternion rotate(float angle, AVector axis)
		{
		float s, c;
		sincos(angle/2.0f, s, c);

		return *this*Quaternion(c, s*x, s*y, s*z);
		}

	AMatrix matrix()
		{
		const float xx =x*x;
		const float xy2=x*y*2.0f;
		const float xz2=x*z*2.0f;
		const float xw2=x*w*2.0f;
		const float yy =y*y;
		const float yz2=y*z*2.0f;
		const float yw2=y*w*2.0f;
		const float zz =z*z;
		const float zw2=z*w*2.0f;
		const float ww =w*w;

		return AMatrix(AVector(ww+xx-yy-zz, xy2-zw2, xz2+wy2, 0.0f),
					   AVector(xy2+zw2, ww-xx+yy-zz, yz2+xw2, 0.0f),
					   AVector(xz2-yw2, yz2-xw2, ww-xx-yy+zz, 0.0f),
					   AVector(0.0f, 0.0f, 0.0f, 1.0f));
		}

	/*Quaternion slerp(const Quaternion q, float t)
		{
		return
		}*/
	};
